#========================================================================================
# §package      DE_EPAGES::Ebay::t::EbayCategories
# §state        public
#----------------------------------------------------------------------------------------
# §description  Package contains tests for EbayCategories
#========================================================================================
package DE_EPAGES::Ebay::t::EbayCategories;
use base DE_EPAGES::Database::API::Test;

use strict;

use DE_EPAGES::Core::API::File qw ( OpenFile );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Database::API::Config qw ( GetSQLiteDBFile );
use DE_EPAGES::Ebay::API::Cartridge;
use DE_EPAGES::Ebay::API::Ebay qw ( UpdateCategories );
use DE_EPAGES::Ebay::API::Constants;
use DE_EPAGES::Ebay::API::Object::EbayCategoryTree;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use Test::More;

our $EBAY_CATEGORIES_DB = GetSQLiteDBFile('EbayCategories') // EBAY_CATEGORIES_DB_FILENAME;

sub init {
    my $self = shift;

    my $Cartridge = DE_EPAGES::Ebay::API::Cartridge->new();
    $Cartridge->installEbayCategories(EBAY_CATEGORIES_DB_NAME, $EBAY_CATEGORIES_DB);

    return;
}

sub test {
    my $self = shift;

    $self->test_UpdateCategories();
    done_testing();

    return;
}

sub test_UpdateCategories {
    my $self = shift;

    my $aEnvironments = LoadRootObject()->get('EbayEnvironments');
    my @ActiveEnvironments = grep {$_->get('IsVisible')} @{$aEnvironments};
    my $EbayEnvironment = $ActiveEnvironments[0];

    plan skip_all => 'Because eBay environment is not available' unless defined $EbayEnvironment;

    UpdateCategories($EbayEnvironment->alias, 0, 0);

    my $EbayCategoryTree = DE_EPAGES::Ebay::API::Object::EbayCategoryTree->new($EbayEnvironment);
    my $ahInfoRootCategories = $EbayCategoryTree->getInfoRootCategories();
    ok( isValid($ahInfoRootCategories->[0]->{'EbayCategoryID'}), 'access root category' );

    $self->_damageSQLiteDB($EBAY_CATEGORIES_DB);

    UpdateCategories($EbayEnvironment->alias, 0, 0);

    $EbayCategoryTree = DE_EPAGES::Ebay::API::Object::EbayCategoryTree->new($EbayEnvironment);
    $ahInfoRootCategories = $EbayCategoryTree->getInfoRootCategories();
    ok( isValid($ahInfoRootCategories->[0]->{'EbayCategoryID'}), 'access root category after repairing db' );

    return;
}

sub _damageSQLiteDB {
    my $self = shift;
    my ($DBFileName) = @_;

    OpenFile( '>', $DBFileName, sub {
            my ($FileHandle) = @_;

            print $FileHandle 'invalid content';
        }
    );

    return;
}

__PACKAGE__->runScript();
1;
