#IF(
    #DEFINED(#Shop.GoogleSmartShoppingConfig)
    AND #DEFINED(#Shop.GoogleSmartShoppingConfig.AdsCampaignConversionId)
    AND #DEFINED(#Shop.GoogleSmartShoppingConfig.AdsCampaignConversionLabel)
    AND #DEFINED(#Shop.GoogleSmartShoppingConfig.AdsCampaignRemarketingId)
)
  (function () {

    const adsCampaignConversionId = "#Shop.GoogleSmartShoppingConfig.AdsCampaignConversionId";
    const adsCampaignConversionLabel = "#Shop.GoogleSmartShoppingConfig.AdsCampaignConversionLabel";
    const adsCampaignRemarketingId = "#Shop.GoogleSmartShoppingConfig.AdsCampaignRemarketingId";

    let googlePortalsLocale = [];

    #LOOP(#Shop.PortalConfigs)
      #IF(#Portal.Alias EQ "GoogleBase")
        googlePortalsLocale.push("#PortalSite.LocaleID")
      #ENDIF
    #ENDLOOP

    function gtag() {
      dataLayer.push(arguments);
    }

    if (! (window.dataLayer && document.getElementById('GAW_googletagmanager'))) {
        let googleSmartShoppingScriptElement = document.createElement('script')
        googleSmartShoppingScriptElement.src = 'https://www.googletagmanager.com/gtag/js?id=' + adsCampaignConversionId
        googleSmartShoppingScriptElement.id = 'GSG_googletagmanager'
        document.body.appendChild(googleSmartShoppingScriptElement)

        window.dataLayer = window.dataLayer || [];
        gtag('js', new Date());
    }

    gtag('config', adsCampaignConversionId);

    return function () {
      if (window.eComEventTarget) {
        window.eComEventTarget.addEventListener('order:completed', onOrderCompletedEvent)
        window.eComEventTarget.addEventListener('cart:add', onCartAddEvent)
      }
    }();

  function onOrderCompletedEvent (event) {
    var order = event.detail
    if (!order || !order.lineItemContainer || order.lineItemContainer.grandTotal.amount <= 0) {
      return
    }

    const eventParameters = {
      'send_to': adsCampaignConversionLabel,
      'transaction_id': order.orderNumber,
      'currency': order.lineItemContainer.grandTotal.currency,
      'value': order.lineItemContainer.grandTotal.amount
    }

    gtag('event', 'conversion', eventParameters)
  }

  function onCartAddEvent (event) {
    var product = event.detail.product
    var eventQuantity = event.detail.quantity

    if (!product || !product.price || product.price.amount <= 0 || eventQuantity <= 0) {
      return
    }

    let googleProduct = getGoogleProduct(product)
    googleProduct['quantity'] = eventQuantity

    const eventParameters = {
      'currency': product.price.currency,
      'value': (googleProduct.price * googleProduct.quantity).toFixed(2),
      'items': [googleProduct]
    }

    gtag('event', 'add_to_cart', eventParameters);
  }

  function getGoogleProduct(product) {
    return {
      'item_id': getGoogleProductId(product.sku),
      'item_name': product.name,
      'price': product.price.amount
    };
  }

  function getGoogleProductId(productCode) {
    if (googlePortalsLocale.length === 0) {
        return productCode;
    }

    if (googlePortalsLocale.length === 1) {
      return (productCode + '_' + googlePortalsLocale[0])
    }

    const shopperLocale = getShopperLocale()
    const matchingPortalsLocale = googlePortalsLocale.filter(portalLocale => portalLocale.includes(shopperLocale))

    if (matchingPortalsLocale.length >= 1) {
      return (productCode + '_' + matchingPortalsLocale[0])
    }

    return (productCode + '_' + googlePortalsLocale[0])
  }

  function getShopperLocale() {
    var countryCode = ''
    if (document.documentElement && document.documentElement.lang) {
      countryCode = document.documentElement.lang
    }
    return countryCode
  }

  }())
#ENDIF
