#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::GoogleApiService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for UI::Provider
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::GoogleAdsApiService;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled GetParametersFromMockifyCall );
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;
use DE_EPAGES::GoogleSmartShopping::API::Constants qw ( EPAGESJ_BASE_URI );

use strict;

use constant AUTH_CODE => 'SOME_AUTH_CODE';

use constant ACCOUNT_EMAIL => 'some_email@example.com';
use constant ADMIN_REFRESH_TOKEN => 'SOME_ADMIN_REFRESH_TOKEN';
use constant ADMIN_ACCESS_TOKEN => 'SOME_ADMIN_ACCESS_TOKEN';
use constant EPAGES_TZ => 'Europe/Berlin';
use constant EPAGES_ADS_DEVELOPER_TOKEN => 'SOME_EPAGES_ADS_DEVELOPER_TOKEN';

use constant APP_CLIENT_ID => 'SomeClientId';
use constant APP_CLIENT_SECRET => 'SomeClientSecret';
use constant AUTH_TOKEN_URL => 'SomeAuthTokenUrl';
use constant APP_REDIRECT_URL => 'SomeAppRedirectUrl';
use constant ADMIN_MULTI_ACCOUNT_ID => 53255422348;
use constant MERCHANT_CENTER_ACCOUNT_ID     => 99039382162;
use constant MERCHANT_CENTER_MULTI_ACCOUNT_ID => 43242342348;
use constant EPAGES_MERCHANT_CENTER_MULTI_ACCOUNT_ID => 7654765478348;
use constant GOOGLE_ADS_ACCOUNT_ID => 9871958046;
use constant GOOGLE_ADS_ACCOUNT_NAME => 'TestShop';
use constant GOOGLE_ADS_MANAGER_ACCOUNT_ID => 12365400001;
use constant GOOGLE_ADS_CAMPAIGN_ID => 9912508531;

use constant EXPECTED_GOOGLE_ADS_ACCOUNTS_RESPONSE => {
    'name' => 'someGoogleAdsAccountName',
    'id' => GOOGLE_ADS_ACCOUNT_ID };
use constant EXPECTED_GOOGLE_ADS_ACCOUNTS_EPAGESJ_URL => 'misc/google-smart-shopping/accounts/ads/list';
use constant EXPECTED_GOOGLE_ADS_ACCOUNTS_CREATION_RESPONSE => {
    'id'   => GOOGLE_ADS_ACCOUNT_ID,
    'name' => => "Some Descriptive Name"
};

use constant EXPECTED_GOOGLE_ADS_CAMPAIGN_RESPONSE => {
    "resourceName"  => "customers/6162055346/campaigns/14810834766",
    'id' => 8065465456546,
};

use constant EXPECTED_GOOGLE_ADS_CAMPAIGN_CONVERSION_ACTION_RESPONSE => {
    'awConversionId' => 'AW-10787257045',
    'awConversionLabel' => 'AW-10787257045/bnmCCNOazv0CENX14Zco',
    'id' => 800296275,
    'name' => 'Spotlight - 9912508531 Conversion Action'
};

use constant EXPECTED_GOOGLE_ADS_CAMPAIGN_REMARKETING_ACTION_RESPONSE => {
    'awRemarketingId' => 'AW-10787257045',
    'id' => 800296275,
    'name' => 'Spotlight - 9912508531 Remarketing Action'
};

use constant EXPECTED_GOOGLE_SMART_SHOPPING_CAMPAIGNS_RESPONSE => [
    {
        'id' => '14886962060',
        'monthlyBudget' => {
            'amount' => 128,
            'currency' => 'EUR',
            'id' => 9606674338,
            'name' => 'Spotlight-Local-Zwei Campaign'
        },
        'name' => 'Spotlight-Local-Zwei Campaign',
        'resourceName' => 'customers/2080416696/campaigns/14886962060',
        'startDate' => '2021-10-07',
        'status' => 'PAUSED'
    }
];

use constant EXPECTED_GOOGLE_ADS_CAMPAIGN_SHOPPING_PERFORMANCE_RESPONSE => {
    'results'     => 45, # total number of available items (not the amount returned)
    'page' => 1,
    'resultsPerPage' => 10,
    'items' => [
        { 'productName' => '01-Lifestyle Headphones', 'clicks' => '54', 'cost' => '14.00', 'conversions' => '2', 'conversionValue' => '35.00' },
        { 'productName' => '02-Navy Boots', 'clicks' => '34', 'cost' => '8.00', 'conversions' => '6', 'conversionValue' => '48.00' },
        { 'productName' => '03-Black Cocktail Dress', 'clicks' => '5', 'cost' => '3.00', 'conversions' => '0', 'conversionValue' => '0.00' },
        { 'productName' => '04-Black Belt', 'clicks' => '14', 'cost' => '4.00', 'conversions' => '4', 'conversionValue' => '25.00' },
        { 'productName' => '05-Brown Boots', 'clicks' => '4', 'cost' => '7.00', 'conversions' => '1', 'conversionValue' => '18.00' },
        { 'productName' => '06-Lifestyle Mouse', 'clicks' => '1', 'cost' => '14.00', 'conversions' => '0', 'conversionValue' => '0.00' },
        { 'productName' => '07-Navy Jacket', 'clicks' => '33', 'cost' => '28.00', 'conversions' => '16', 'conversionValue' => '41.00' },
        { 'productName' => '08-Black Running Pants', 'clicks' => '2', 'cost' => '3.00', 'conversions' => '0', 'conversionValue' => '0.00' },
        { 'productName' => '09-Black Shoes', 'clicks' => '12', 'cost' => '14.00', 'conversions' => '4', 'conversionValue' => '26.00' },
        { 'productName' => '10-Brown Jacket', 'clicks' => '38', 'cost' => '27.00', 'conversions' => '9', 'conversionValue' => '38.00' },
    ]
};

use constant EXPECTED_GOOGLE_ADS_CAMPAIGN_EPAGESJ_URL =>
    "misc/google-smart-shopping/accounts/ads/" . GOOGLE_ADS_MANAGER_ACCOUNT_ID . "/sub-accounts/" . GOOGLE_ADS_ACCOUNT_ID . "/smart-shopping-campaigns";

use constant EXPECTED_GOOGLE_ADS_CAMPAIGN_CONVERSION_ACTION_EPAGESJ_URL =>
    EXPECTED_GOOGLE_ADS_CAMPAIGN_EPAGESJ_URL . '/conversion-actions';

use constant EXPECTED_GOOGLE_ADS_CAMPAIGN_REMARKETING_ACTION_EPAGESJ_URL =>
    EXPECTED_GOOGLE_ADS_CAMPAIGN_EPAGESJ_URL . '/remarketing-actions';

use constant EXPECTED_GOOGLE_ADS_CAMPAIGN_UPDATE_BUDGET_EPAGESJ_URL =>
    EXPECTED_GOOGLE_ADS_CAMPAIGN_EPAGESJ_URL . '/' . GOOGLE_ADS_CAMPAIGN_ID . '/daily-budget';

use constant EXPECTED_GOOGLE_ADS_CAMPAIGN_SWITCH_STATUS_EPAGESJ_URL =>
    EXPECTED_GOOGLE_ADS_CAMPAIGN_EPAGESJ_URL . '/' . GOOGLE_ADS_CAMPAIGN_ID ;

use constant EXPECTED_GOOGLE_ADS_ACCOUNT_DELETE_EPAGESJ_URL =>
    "misc/google-smart-shopping/accounts/ads/" . GOOGLE_ADS_MANAGER_ACCOUNT_ID . "/sub-accounts/" . GOOGLE_ADS_ACCOUNT_ID;

use constant EXPECTED_GOOGLE_ADS_CAMPAIGN_SHOPPING_PERFORMANCE_EPAGESJ_URL =>
    EXPECTED_GOOGLE_ADS_CAMPAIGN_EPAGESJ_URL . '/' . GOOGLE_ADS_CAMPAIGN_ID . '/shopping-performance';

use constant TEST_ADDRESS_COUNTRY_NAME => "Germany";
use constant TEST_ADDRESS_COUNTRY_ISO_CODE => "DE";

use constant ADS_ACCOUNT_BILLING_STATUS_APPROVED => 'APPROVED';
use constant ADS_ACCOUNT_BILLING_STATUS_UNAPPROVED => 'UNKNOWN';
use constant BAD_REQUEST_ERROR_RESPONSE => {
    "errorId" => "google-ads-api-error",
    "message" => "Error on attempt to interact with Google Ads API",
    "details" => "errors {\n  error_code {\n    manager_link_error: CLIENT_HAS_NO_ADMIN_USER\n  }\n  message: \"Client has no active user that can access the client account.\"\n  location {\n    field_path_elements {\n      field_name: \"operations\"\n      index: 0\n    }\n  }\n}\nrequest_id: \"30dsjbT_n-SWy5Tx8vm1Yw\"\n"
};

sub test {
    my $self = shift;

    $self->test_get_google_ads_accounts_success();
    $self->test_get_google_ads_accounts_no_accounts();
    $self->test_create_Ads_Account_success();
    $self->test_create_Link_Ads_Account_success_epages_created_account();
    $self->test_create_Link_Ads_Account_success_merchant_created_account();
    $self->test_enable_Link_Ads_Account_success();
    $self->test_create_ads_campaign_success();
    $self->test_switch_ads_campaign_status_success();
    $self->test_update_ads_campaign_budget_success();
    $self->test_create_ads_campaign_conversion_action_success();
    $self->test_create_ads_campaign_remarketing_action_success();
    $self->test_get_ads_campaign_success();
    $self->test_get_billing_status_approved_success();
    $self->test_get_billing_status_unapproved_success();
    $self->test_is_billing_approved_success();
    $self->test_is_billing_unapproved_success();
    $self->test_delete_Ads_Account_success();
    $self->test_delete_Ads_Account_failure();
    $self->test_delete_ads_campaign();
    $self->test_get_campaign_shopping_performance();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------

sub test_get_google_ads_accounts_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    lives_ok(
        sub { $SUT = $self->_createSUT({
            'UserAgent' => $MockedUserAgent,
            'DoGetResponse' => EXPECTED_GOOGLE_ADS_ACCOUNTS_RESPONSE}); },
        "$SubTestName - Service initialization works."
    );

    lives_ok(
        sub { $Response = $SUT->getGoogleAdsAccounts(); },
        "$SubTestName - Verify that getGoogleAdsAccounts can be called without errors."
    );

    ok(
        WasCalled($SUT, 'doGet'),
        $SubTestName . ' - Verify that doGet was called.'
    );

    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doGet');
    is( @$aGetParams[0], EXPECTED_GOOGLE_ADS_ACCOUNTS_EPAGESJ_URL, "$SubTestName - Verify doGet call correctly the epagesj endpoint." );
    is( $Response, EXPECTED_GOOGLE_ADS_ACCOUNTS_RESPONSE, "$SubTestName - Verify that Google Ads Accounts list correctly returned." );
}

sub test_get_google_ads_accounts_no_accounts {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    lives_ok(
        sub { $SUT = $self->_createSUT({
            'UserAgent' => $MockedUserAgent,
            'DoGetResponse' => []
        }); },
        "$SubTestName - Service initialization works."
    );

    lives_ok(
        sub { $Response = $SUT->getGoogleAdsAccounts(); },
        "$SubTestName - getGoogleAdsAccounts can be called without errors."
    );
    is_deeply( $Response, [], "$SubTestName - Google Ads Accounts empty list correctly returned." );
}

sub test_create_Ads_Account_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    lives_ok(
        sub { $SUT = $self->_createSUT({
            'UserAgent' => $MockedUserAgent,
            'DoPostResponse' => EXPECTED_GOOGLE_ADS_ACCOUNTS_CREATION_RESPONSE,
            'CurrencyID' => 'EUR'});
             },
        "$SubTestName - Service initialization works."
    );

    $SUT->{'GoogleSmartShoppingConfig'} = $self->_createGoogleConfig({
        GoogleAdsAccountID => undef,
        GoogleAdsAccountName => undef,
        IsAdsAccountSubAccount => 0
    });

    lives_ok(
        sub { $Response = $SUT->createGoogleAdsSubAccount(); },
        "$SubTestName - createGoogleAdsSubAccount can be called without errors."
    );


    ok( WasCalled($SUT, '_getAdsAccountName'), $SubTestName . ' - Verify that _getAdsAccountName was called.');
    ok( WasCalled($SUT, '_getAdsManagerAccountId'), $SubTestName . ' - Verify that _getAdsManagerAccountId was called.');
    ok( WasCalled($SUT, '_getEmail'), $SubTestName . ' - Verify that _getEmail was called.');
    ok( WasCalled($SUT, '_createAdminToken'), $SubTestName . ' - Verify that _createAdminToken was called.');
    ok( WasCalled($SUT, '_getDeveloperToken'), $SubTestName . ' - Verify that _getDeveloperToken was called.');
    ok( WasCalled($SUT, '_getCurrencyCode'), $SubTestName . ' - Verify that _getCurrencyCode was called.');
    ok( WasCalled($SUT, '_getTimeZone'), $SubTestName . ' - Verify that _getTimeZone was called.');
    ok( WasCalled($SUT, 'doPost'), $SubTestName . ' - Verify that doPost was called.');

    is(
        $SUT->{'GoogleSmartShoppingConfig'}->get('GoogleAdsAccountID'),
        EXPECTED_GOOGLE_ADS_ACCOUNTS_CREATION_RESPONSE->{'id'},
        $SubTestName . ' - Verify that GoogleAdsAccountID was saved.'
    );
    is( $SUT->{'GoogleSmartShoppingConfig'}->get('GoogleAdsAccountID'), GOOGLE_ADS_ACCOUNT_ID, $SubTestName . ' - Verify that GoogleAdsAccountID was saved.');
    is( $SUT->{'GoogleSmartShoppingConfig'}->get('GoogleAdsAccountName'), GOOGLE_ADS_ACCOUNT_NAME, $SubTestName . ' - Verify that GoogleAdsAccountName was saved.');
    is( $SUT->{'GoogleSmartShoppingConfig'}->get('IsAdsAccountSubAccount'), 1, $SubTestName . ' - Verify that GoogleAdsAccountID was saved.');
}

sub test_create_Link_Ads_Account_success_epages_created_account {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    $SUT = $self->_createSUT({
        'UserAgent' => $MockedUserAgent,
        'DoPutResponse' => '',
    });

    $SUT->{'GoogleSmartShoppingConfig'} = $self->_createGoogleConfig({
        GoogleAdsAccountID => GOOGLE_ADS_ACCOUNT_ID,
        MerchantCenterAccountID => MERCHANT_CENTER_ACCOUNT_ID,
        MerchantCenterMultiClientAccountID => EPAGES_MERCHANT_CENTER_MULTI_ACCOUNT_ID,
    });

    $SUT->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'} = EPAGES_MERCHANT_CENTER_MULTI_ACCOUNT_ID;

    lives_ok(
        sub { $Response = $SUT->createGoogleAdsLink(); },
        "$SubTestName - createGoogleAdsLink can be called without errors."
    );

    ok( WasCalled($SUT, 'doPut'), $SubTestName . ' - Verify that doPut was called.');

    my $ExpectedUrl = sprintf("%saccounts/%s/sub-accounts/%s/ads-link" ,
        EPAGESJ_BASE_URI,
        EPAGES_MERCHANT_CENTER_MULTI_ACCOUNT_ID,
        MERCHANT_CENTER_ACCOUNT_ID
    );

    my $hExpectedHeaders = {
        AdminToken =>  ADMIN_ACCESS_TOKEN,
    };

    my $hExpectedPayload = {
        adsAccountId =>  GOOGLE_ADS_ACCOUNT_ID,
    };

    my $aPutParams = GetParametersFromMockifyCall($SUT, 'doPut');

    is( $aPutParams->[0], $ExpectedUrl, "$SubTestName - Verify doPut Url parameter is correct." );
    is_deeply( $aPutParams->[1], $hExpectedHeaders, "$SubTestName - Verify doPut Headers parameter is correct." );
    is_deeply( $aPutParams->[2], $hExpectedPayload, "$SubTestName - Verify doPut Payload parameter is correct." );
}

sub test_create_Link_Ads_Account_success_merchant_created_account {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    $SUT = $self->_createSUT({
        'UserAgent' => $MockedUserAgent,
        'DoPutResponse' => '',
    });

    $SUT->{'GoogleSmartShoppingConfig'} = $self->_createGoogleConfig({
        GoogleAdsAccountID => GOOGLE_ADS_ACCOUNT_ID,
        MerchantCenterAccountID => MERCHANT_CENTER_ACCOUNT_ID,
        MerchantCenterMultiClientAccountID => undef,
    });

    $SUT->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'} = EPAGES_MERCHANT_CENTER_MULTI_ACCOUNT_ID;

    lives_ok(
        sub { $Response = $SUT->createGoogleAdsLink(); },
        "$SubTestName - createGoogleAdsLink can be called without errors."
    );

    ok( WasCalled($SUT, 'doPut'), $SubTestName . ' - Verify that doPut was called.');

    my $ExpectedUrl = sprintf("%saccounts/%s/sub-accounts/%s/ads-link" ,
        EPAGESJ_BASE_URI,
        MERCHANT_CENTER_ACCOUNT_ID,
        MERCHANT_CENTER_ACCOUNT_ID
    );

    my $hExpectedHeaders = {};

    my $hExpectedPayload = {
        adsAccountId =>  GOOGLE_ADS_ACCOUNT_ID,
    };

    my $aPutParams = GetParametersFromMockifyCall($SUT, 'doPut');
    is( $aPutParams->[0], $ExpectedUrl, "$SubTestName - Verify doPut Url parameter is correct." );
    is_deeply( $aPutParams->[1], $hExpectedHeaders, "$SubTestName - Verify doPut Headers parameter is correct." );
    is_deeply( $aPutParams->[2], $hExpectedPayload, "$SubTestName - Verify doPut Payload parameter is correct." );
}


sub test_enable_Link_Ads_Account_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    $SUT = $self->_createSUT({
        'UserAgent' => $MockedUserAgent,
        'DoPostResponse' => '',
    });

    $SUT->{'GoogleSmartShoppingConfig'} = $self->_createGoogleConfig({
        GoogleAdsAccountID => GOOGLE_ADS_ACCOUNT_ID,
        MerchantCenterAccountID => MERCHANT_CENTER_ACCOUNT_ID,
        MerchantCenterMultiClientAccountID => EPAGES_MERCHANT_CENTER_MULTI_ACCOUNT_ID,
    });

    $SUT->{'GoogleSmartShopping_AdsManagerAccountId'} = GOOGLE_ADS_MANAGER_ACCOUNT_ID;

    lives_ok(
        sub { $Response = $SUT->enableGoogleAdsLink(); },
        "$SubTestName - enableGoogleAdsLink can be called without errors."
    );

    ok( WasCalled($SUT, 'doPost'), $SubTestName . ' - Verify that doPut was called.');

    my $ExpectedUrl = sprintf("%saccounts/ads/%s/sub-accounts/%s/merchant-center-link/enable" ,
        EPAGESJ_BASE_URI,
        GOOGLE_ADS_MANAGER_ACCOUNT_ID,
        GOOGLE_ADS_ACCOUNT_ID
    );

    my $hExpectedHeaders = {
        'AdminToken' =>  ADMIN_ACCESS_TOKEN,
        'X-DeveloperToken' => EPAGES_ADS_DEVELOPER_TOKEN,
    };

    my $hExpectedPayload = {};

    my $aPutParams = GetParametersFromMockifyCall($SUT, 'doPost');

    is( $aPutParams->[0], $ExpectedUrl, "$SubTestName - Verify doPut Url parameter is correct." );
    is_deeply( $aPutParams->[1], $hExpectedHeaders, "$SubTestName - Verify doPut Headers parameter is correct." );
    is_deeply( $aPutParams->[2], $hExpectedPayload, "$SubTestName - Verify doPut Payload parameter is correct." );
}

sub test_create_ads_campaign_success {
    my $self =shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    my $MockedShoppingConfig = $self->_createGoogleConfig({
        GoogleAdsAccountID => GOOGLE_ADS_ACCOUNT_ID,
        MerchantCenterAccountID => MERCHANT_CENTER_ACCOUNT_ID,
        MerchantCenterMultiClientAccountID => EPAGES_MERCHANT_CENTER_MULTI_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);


    lives_ok(
        sub { $SUT = $self->_createSUT({
            'Shop' => $MockedShop,
            'UserAgent' => $MockedUserAgent,
            'DoPostResponse' => EXPECTED_GOOGLE_ADS_CAMPAIGN_RESPONSE}); },
        "$SubTestName - Service initialization works."
    );

    my $hParams = {
        'AdsSubAccountId'         => GOOGLE_ADS_ACCOUNT_ID,
        'MerchantCenterAccountId' => MERCHANT_CENTER_ACCOUNT_ID,
        'DailyBudget'             => 12.34,
        'AdsCampaignName'         => 'CubaMissileCrisis',
    };

    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;

    lives_ok(
        sub { $Response = $SUT->createGoogleAdsCampaign($hParams); },
        "$SubTestName - Verify that createGoogleAdsCampaign can be called without errors."
    );

    ok(
        WasCalled($SUT, 'doPost'),
        $SubTestName . ' - Verify that doPost was called.'
    );


    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doPost');
    is( @$aGetParams[0], EXPECTED_GOOGLE_ADS_CAMPAIGN_EPAGESJ_URL, "$SubTestName - Verify doPost call correctly the epagesj endpoint." );
    is( $Response, EXPECTED_GOOGLE_ADS_CAMPAIGN_RESPONSE, "$SubTestName - Verify that Google Ads created campaign is correctly returned." );
    is( $MockedShoppingConfig->get('AdsCampaignId'), EXPECTED_GOOGLE_ADS_CAMPAIGN_RESPONSE->{'id'}, "$SubTestName - Verify that created campaign id is correctly saved." );
}

sub test_switch_ads_campaign_status_success {
    my $self =shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;

    my $MockedShoppingConfig = $self->_createGoogleConfig({
        GoogleAdsAccountID => GOOGLE_ADS_ACCOUNT_ID,
        MerchantCenterAccountID => MERCHANT_CENTER_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);

    lives_ok(
        sub { $SUT = $self->_createSUT({
            'Shop' => $MockedShop,
            'UserAgent' => $MockedUserAgent,
            'DoPutResponse' => '' }); },
        "$SubTestName - Service initialization works."
    );

    my $hParams = {
        'CampaignId' => GOOGLE_ADS_CAMPAIGN_ID,
        'CampaignStatusEnabled' => 0,
    };
    lives_ok(
        sub { $SUT->updateGoogleAdsCampaign($hParams); },
        "$SubTestName - Verify that updateGoogleAdsCampaign can be called without errors."
    );

    ok(
        WasCalled($SUT, 'doPut'),
        $SubTestName . ' - Verify that doPut was called.'
    );

    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doPut');
    is(
        @$aGetParams[0],
        EXPECTED_GOOGLE_ADS_CAMPAIGN_SWITCH_STATUS_EPAGESJ_URL,
        "$SubTestName - Verify doPut correctly calls the epagesj endpoint."
    );

}

sub test_update_ads_campaign_budget_success {
    my $self =shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;

    my $MockedShoppingConfig = $self->_createGoogleConfig({
        GoogleAdsAccountID => GOOGLE_ADS_ACCOUNT_ID,
        MerchantCenterAccountID => MERCHANT_CENTER_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);

    lives_ok(
        sub { $SUT = $self->_createSUT({
            'Shop' => $MockedShop,
            'UserAgent' => $MockedUserAgent,
            'DoPutResponse' => '' }); },
        "$SubTestName - Service initialization works."
    );

    my $hParams = {
        'AdsCampaignId' => GOOGLE_ADS_CAMPAIGN_ID,
        'DailyBudget' => 12.34,
        'DailyBudgetCurrency' => 'EUR',
    };
    lives_ok(
        sub { $SUT->updateGoogleAdsCampaignBudget($hParams); },
        "$SubTestName - Verify that updateGoogleAdsCampaignBudget can be called without errors."
    );

    ok(
        WasCalled($SUT, 'doPut'),
        $SubTestName . ' - Verify that doPut was called.'
    );

    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doPut');
    is(
        @$aGetParams[0],
        EXPECTED_GOOGLE_ADS_CAMPAIGN_UPDATE_BUDGET_EPAGESJ_URL,
        "$SubTestName - Verify doPut correctly calls the epagesj endpoint."
    );

}


sub test_create_ads_campaign_conversion_action_success {
    my $self =shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    my $MockedShoppingConfig = $self->_createGoogleConfig({
        GoogleAdsAccountID => GOOGLE_ADS_ACCOUNT_ID,
        MerchantCenterAccountID => MERCHANT_CENTER_ACCOUNT_ID,
        MerchantCenterMultiClientAccountID => EPAGES_MERCHANT_CENTER_MULTI_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);


    lives_ok(
        sub { $SUT = $self->_createSUT({
            'Shop' => $MockedShop,
            'UserAgent' => $MockedUserAgent,
            'DoPostResponse' => EXPECTED_GOOGLE_ADS_CAMPAIGN_CONVERSION_ACTION_RESPONSE}); },
        "$SubTestName - Service initialization works."
    );

    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;

    lives_ok(
        sub { $Response = $SUT->createGoogleConversionAction(GOOGLE_ADS_CAMPAIGN_ID); },
        "$SubTestName - Verify that createGoogleConversionAction can be called without errors."
    );

    ok(
        WasCalled($SUT, 'doPost'),
        $SubTestName . ' - Verify that doPost was called.'
    );

    my $hExpectedPayload = {
        name =>  'ShopNameOrAlias - ' . GOOGLE_ADS_CAMPAIGN_ID . ' Conversion Action',
    };
    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doPost');
    is( $aGetParams->[0], EXPECTED_GOOGLE_ADS_CAMPAIGN_CONVERSION_ACTION_EPAGESJ_URL, "$SubTestName - Verify doPost call correctly the epagesj endpoint." );
    is_deeply( $aGetParams->[2], $hExpectedPayload, "$SubTestName - Verify doPost call has correct payload." );
    is( $Response, EXPECTED_GOOGLE_ADS_CAMPAIGN_CONVERSION_ACTION_RESPONSE, "$SubTestName - Verify that Google Ads created campaign conversion action is correctly returned." );

    is( 
        $SUT->{'GoogleSmartShoppingConfig'}->get('AdsCampaignConversionId'),
        EXPECTED_GOOGLE_ADS_CAMPAIGN_CONVERSION_ACTION_RESPONSE->{'awConversionId'},
        "$SubTestName - Verify AdsCampaignConversionId gets set correctly."
    );

    is( 
        $SUT->{'GoogleSmartShoppingConfig'}->get('AdsCampaignConversionLabel'),
        EXPECTED_GOOGLE_ADS_CAMPAIGN_CONVERSION_ACTION_RESPONSE->{'awConversionLabel'},
        "$SubTestName - Verify AdsCampaignConversionLabel gets set correctly."
    );
}

sub test_create_ads_campaign_remarketing_action_success {
    my $self =shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    my $MockedShoppingConfig = $self->_createGoogleConfig({
        GoogleAdsAccountID => GOOGLE_ADS_ACCOUNT_ID,
        MerchantCenterAccountID => MERCHANT_CENTER_ACCOUNT_ID,
        MerchantCenterMultiClientAccountID => EPAGES_MERCHANT_CENTER_MULTI_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);


    lives_ok(
        sub { $SUT = $self->_createSUT({
            'Shop' => $MockedShop,
            'UserAgent' => $MockedUserAgent,
            'DoPostResponse' => EXPECTED_GOOGLE_ADS_CAMPAIGN_REMARKETING_ACTION_RESPONSE}); },
        "$SubTestName - Service initialization works."
    );

    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;

    lives_ok(
        sub { $Response = $SUT->createGoogleRemarketingAction(GOOGLE_ADS_CAMPAIGN_ID); },
        "$SubTestName - Verify that createGoogleRemarketingAction can be called without errors."
    );

    ok(
        WasCalled($SUT, 'doPost'),
        $SubTestName . ' - Verify that doPost was called.'
    );

    my $hExpectedPayload = {
        name =>  'ShopNameOrAlias - ' . GOOGLE_ADS_CAMPAIGN_ID . ' Remarketing Action',
    };
    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doPost');
    is( @$aGetParams[0], EXPECTED_GOOGLE_ADS_CAMPAIGN_REMARKETING_ACTION_EPAGESJ_URL, "$SubTestName - Verify doPost call correctly the epagesj endpoint." );
    is_deeply( $aGetParams->[2], $hExpectedPayload, "$SubTestName - Verify doPost call has correct payload." );
    is( $Response, EXPECTED_GOOGLE_ADS_CAMPAIGN_REMARKETING_ACTION_RESPONSE, "$SubTestName - Verify that Google Ads created campaign remarketing action is correctly returned." );

    is( 
        $SUT->{'GoogleSmartShoppingConfig'}->get('AdsCampaignRemarketingId'),
        EXPECTED_GOOGLE_ADS_CAMPAIGN_REMARKETING_ACTION_RESPONSE->{'awRemarketingId'},
        "$SubTestName - Verify AdsCampaignRemarketingId gets set correctly."
    );
}


sub test_get_ads_campaign_success {
    my $self =shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    my $MockedShoppingConfig = $self->_createGoogleConfig({
        GoogleAdsAccountID => GOOGLE_ADS_ACCOUNT_ID,
        MerchantCenterAccountID => MERCHANT_CENTER_ACCOUNT_ID,
        MerchantCenterMultiClientAccountID => EPAGES_MERCHANT_CENTER_MULTI_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);


    $SUT = $self->_createSUT({
            'Shop' => $MockedShop,
            'UserAgent' => $MockedUserAgent,
            'DoGetResponse' => EXPECTED_GOOGLE_SMART_SHOPPING_CAMPAIGNS_RESPONSE});

    my $hParams = {
        'AdsSubAccountId'         => GOOGLE_ADS_ACCOUNT_ID
    };
    lives_ok(
        sub { $Response = $SUT->getSmartShoppingCampaigns($hParams); },
        "$SubTestName - Verify that createGoogleAdsCampaign can be called without errors."
    );

    ok(WasCalled($SUT, '_getAdsManagerAccountId'), $SubTestName . ' - Verify that doPost was called.');
    ok(WasCalled($SUT, '_createAdminToken'), $SubTestName . ' - Verify that _createAdminToken was called.');
    ok(WasCalled($SUT, '_getDeveloperToken'), $SubTestName . ' - Verify that _getDeveloperToken was called.');
    ok(WasCalled($SUT, 'doGet'), $SubTestName . ' - Verify that doGet was called.');

    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doGet');
    is( @$aGetParams[0], EXPECTED_GOOGLE_ADS_CAMPAIGN_EPAGESJ_URL, "$SubTestName - Verify doGet call correctly the epagesj endpoint." );
    is( $Response, EXPECTED_GOOGLE_SMART_SHOPPING_CAMPAIGNS_RESPONSE, "$SubTestName - Verify that Google Smart Shopping Campaigns list correctly returned." );

}

sub test_get_billing_status_approved_success {
    my $self =shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    my $MockedShoppingConfig = $self->_createGoogleConfig({
        GoogleAdsAccountID => GOOGLE_ADS_ACCOUNT_ID,
        MerchantCenterAccountID => MERCHANT_CENTER_ACCOUNT_ID,
        MerchantCenterMultiClientAccountID => EPAGES_MERCHANT_CENTER_MULTI_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);

    $SUT = $self->_createSUT({
            'Shop' => $MockedShop,
            'UserAgent' => $MockedUserAgent,
            'DoGetResponse' => {
                status => ADS_ACCOUNT_BILLING_STATUS_APPROVED
            }});

    lives_ok(
        sub { $Response = $SUT->getAdsAccountBillingStatus(); },
        "$SubTestName - Verify that getAdsAccountBillingStatus can be called without errors."
    );

    ok(WasCalled($SUT, '_createAdminToken'), $SubTestName . ' - Verify that _createAdminToken was called.');
    ok(WasCalled($SUT, '_getDeveloperToken'), $SubTestName . ' - Verify that _getDeveloperToken was called.');
    ok(WasCalled($SUT, 'doGet'), $SubTestName . ' - Verify that doGet was called.');

    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doGet');
    is(
        @$aGetParams[0],
        sprintf("%saccounts/ads/%s/sub-accounts/%s/billing-setup",
            EPAGESJ_BASE_URI,
            GOOGLE_ADS_MANAGER_ACCOUNT_ID,
            GOOGLE_ADS_ACCOUNT_ID
        ),
        "$SubTestName - Verify doGet call correctly the epagesj endpoint."
    );

    is( $Response, ADS_ACCOUNT_BILLING_STATUS_APPROVED, "$SubTestName - Verify that billing status is returned." );
}

sub test_get_billing_status_unapproved_success {
    my $self =shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    my $MockedShoppingConfig = $self->_createGoogleConfig({
        GoogleAdsAccountID => GOOGLE_ADS_ACCOUNT_ID,
        MerchantCenterAccountID => MERCHANT_CENTER_ACCOUNT_ID,
        MerchantCenterMultiClientAccountID => EPAGES_MERCHANT_CENTER_MULTI_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);

    $SUT = $self->_createSUT({
            'Shop' => $MockedShop,
            'UserAgent' => $MockedUserAgent,
            'DoGetResponse' => {
                status => ADS_ACCOUNT_BILLING_STATUS_UNAPPROVED
            }});

    lives_ok(
        sub { $Response = $SUT->getAdsAccountBillingStatus(); },
        "$SubTestName - Verify that getAdsAccountBillingStatus can be called without errors."
    );

    ok(WasCalled($SUT, '_createAdminToken'), $SubTestName . ' - Verify that _createAdminToken was called.');
    ok(WasCalled($SUT, '_getDeveloperToken'), $SubTestName . ' - Verify that _getDeveloperToken was called.');
    ok(WasCalled($SUT, 'doGet'), $SubTestName . ' - Verify that doGet was called.');

    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doGet');
    is(
        @$aGetParams[0],
        sprintf("%saccounts/ads/%s/sub-accounts/%s/billing-setup",
            EPAGESJ_BASE_URI,
            GOOGLE_ADS_MANAGER_ACCOUNT_ID,
            GOOGLE_ADS_ACCOUNT_ID
        ),
        "$SubTestName - Verify doGet call correctly the epagesj endpoint."
    );

    is( $Response, ADS_ACCOUNT_BILLING_STATUS_UNAPPROVED, "$SubTestName - Verify that billing status is returned." );
}

sub test_is_billing_approved_success {
    my $self =shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    my $MockedShoppingConfig = $self->_createGoogleConfig({
        GoogleAdsAccountID => GOOGLE_ADS_ACCOUNT_ID,
        MerchantCenterAccountID => MERCHANT_CENTER_ACCOUNT_ID,
        MerchantCenterMultiClientAccountID => EPAGES_MERCHANT_CENTER_MULTI_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);

    $SUT = $self->_createSUT({
            'Shop' => $MockedShop,
            'UserAgent' => $MockedUserAgent,
            'DoGetResponse' => {
                status => ADS_ACCOUNT_BILLING_STATUS_APPROVED
            }});

    lives_ok(
        sub { $Response = $SUT->isAdsAccountBillingApproved(); },
        "$SubTestName - Verify that getAdsAccountBillingStatus can be called without errors."
    );

    ok(WasCalled($SUT, '_createAdminToken'), $SubTestName . ' - Verify that _createAdminToken was called.');
    ok(WasCalled($SUT, '_getDeveloperToken'), $SubTestName . ' - Verify that _getDeveloperToken was called.');
    ok(WasCalled($SUT, 'doGet'), $SubTestName . ' - Verify that doGet was called.');

    ok( $Response, "$SubTestName - Verify that isAdsAccountBillingApproved is true." );
}

sub test_is_billing_unapproved_success {
    my $self =shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    my $MockedShoppingConfig = $self->_createGoogleConfig({
        GoogleAdsAccountID => GOOGLE_ADS_ACCOUNT_ID,
        MerchantCenterAccountID => MERCHANT_CENTER_ACCOUNT_ID,
        MerchantCenterMultiClientAccountID => EPAGES_MERCHANT_CENTER_MULTI_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);

    $SUT = $self->_createSUT({
            'Shop' => $MockedShop,
            'UserAgent' => $MockedUserAgent,
            'DoGetResponse' => {
                status => ADS_ACCOUNT_BILLING_STATUS_UNAPPROVED
            }});

    lives_ok(
        sub { $Response = $SUT->isAdsAccountBillingApproved(); },
        "$SubTestName - Verify that getAdsAccountBillingStatus can be called without errors."
    );

    ok(WasCalled($SUT, '_createAdminToken'), $SubTestName . ' - Verify that _createAdminToken was called.');
    ok(WasCalled($SUT, '_getDeveloperToken'), $SubTestName . ' - Verify that _getDeveloperToken was called.');
    ok(WasCalled($SUT, 'doGet'), $SubTestName . ' - Verify that doGet was called.');

    ok( !$Response, "$SubTestName - Verify that isAdsAccountBillingApproved is false." );
}

sub test_delete_Ads_Account_success {
    my $self =shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    my $MockedShoppingConfig = $self->_createGoogleConfig({
        GoogleAdsAccountID => GOOGLE_ADS_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);


    lives_ok(
        sub { $SUT = $self->_createSUT({
            'Shop' => $MockedShop,
            'UserAgent' => $MockedUserAgent,
            'DoDeleteResponse' => {}
        }); },
        "$SubTestName - Service initialization works."
    );

    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;

    lives_ok(
        sub { $Response = $SUT->unlinkGoogleAdsAccount(); },
        "$SubTestName - Verify that unlinkGoogleAdsAccount can be called without errors."
    );

    ok(
        WasCalled($SUT, 'doDelete'),
        $SubTestName . ' - Verify that doDelete was called in the service.'
    );


    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doDelete');
    is( @$aGetParams[0], EXPECTED_GOOGLE_ADS_ACCOUNT_DELETE_EPAGESJ_URL, "$SubTestName - Verify doDelete call correctly the epagesj endpoint." );
    is( $Response, undef, "$SubTestName - Verify that delete Google Ads account response is correctly returned." );

}

sub test_delete_Ads_Account_failure {
    my $self =shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    my $MockedShoppingConfig = $self->_createGoogleConfig({
        GoogleAdsAccountID => GOOGLE_ADS_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);


    lives_ok(
        sub { $SUT = $self->_createSUT({
            'Shop' => $MockedShop,
            'UserAgent' => $MockedUserAgent,
            'DoDeleteError' => BAD_REQUEST_ERROR_RESPONSE,
        }); },
        "$SubTestName - Service initialization works."
    );

    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;

    throws_ok(
        sub { $Response = $SUT->unlinkGoogleAdsAccount(); },
        qr/fail/,
        "$SubTestName - Verify that unlinkGoogleAdsAccount does not catch a thrown error."
    );

    ok(
        WasCalled($SUT, 'doDelete'),
        $SubTestName . ' - Verify that doDelete was called in the service.'
    );


    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doDelete');
    is( @$aGetParams[0], EXPECTED_GOOGLE_ADS_ACCOUNT_DELETE_EPAGESJ_URL, "$SubTestName - Verify doDelete call correctly the epagesj endpoint." );

}

sub test_delete_ads_campaign {
    my $self =shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    my $MockedShoppingConfig = $self->_createGoogleConfig({
        GoogleAdsAccountID => GOOGLE_ADS_ACCOUNT_ID,
        MerchantCenterAccountID => MERCHANT_CENTER_ACCOUNT_ID,
        MerchantCenterMultiClientAccountID => EPAGES_MERCHANT_CENTER_MULTI_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);
        
    $SUT = $self->_createSUT({
        'Shop' => $MockedShop,
        'UserAgent' => $MockedUserAgent,
        'DoDeleteResponse' => {}
    });

    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;

    lives_ok(
        sub { $Response = $SUT->deleteAdsCampaign(GOOGLE_ADS_CAMPAIGN_ID); },
        "$SubTestName - Verify that deleteAdsCampaign can be called without errors."
    );

    ok(WasCalled($SUT, '_createAdminToken'), $SubTestName . ' - Verify that _createAdminToken was called.');
    ok(WasCalled($SUT, '_getDeveloperToken'), $SubTestName . ' - Verify that _getDeveloperToken was called.');
    ok(WasCalled($SUT, 'doDelete'), $SubTestName . ' - Verify that doDelete was called.');

    my $aDeleteParams = GetParametersFromMockifyCall($SUT, 'doDelete');
    is(
        $aDeleteParams->[0],
        sprintf("%saccounts/ads/%s/sub-accounts/%s/smart-shopping-campaigns/%s",
            EPAGESJ_BASE_URI,
            GOOGLE_ADS_MANAGER_ACCOUNT_ID,
            GOOGLE_ADS_ACCOUNT_ID,
            GOOGLE_ADS_CAMPAIGN_ID
        ),
        "$SubTestName - Verify doDelete call correctly the epagesj endpoint."
    );
}

sub test_get_campaign_shopping_performance {
    my $self =shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = undef;
    my $SUT;
    my $Response;

    my $MockedShoppingConfig = $self->_createGoogleConfig({
        GoogleAdsAccountID => GOOGLE_ADS_ACCOUNT_ID,
        MerchantCenterAccountID => MERCHANT_CENTER_ACCOUNT_ID,
        MerchantCenterMultiClientAccountID => EPAGES_MERCHANT_CENTER_MULTI_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);


    $SUT = $self->_createSUT({
        'Shop' => $MockedShop,
        'UserAgent' => $MockedUserAgent,
        'DoGetResponse' => EXPECTED_GOOGLE_ADS_CAMPAIGN_SHOPPING_PERFORMANCE_RESPONSE});

    my $Page = 1;
    my $ResultsPerPage = 10;

    lives_ok(
        sub { $Response = $SUT->getCampaignShoppingPerformance(GOOGLE_ADS_CAMPAIGN_ID, $Page, $ResultsPerPage); },
        "$SubTestName - Verify that getCampaignShoppingPerformance can be called without errors."
    );

    ok(WasCalled($SUT, '_getAdsManagerAccountId'), $SubTestName . ' - Verify that Ads manager account ID is gotten.');
    ok(WasCalled($SUT, '_getAdsAccountId'), $SubTestName . ' - Verify that Ads account ID is gotten.');
    ok(WasCalled($SUT, '_createAdminToken'), $SubTestName . ' - Verify that a new admin token is being generated.');
    ok(WasCalled($SUT, '_getDeveloperToken'), $SubTestName . ' - Verify that developer token is gotten.');
    ok(WasCalled($SUT, 'doGet'), $SubTestName . ' - Verify that doGet was called.');

    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doGet');
    is(
        @$aGetParams[0],
        EXPECTED_GOOGLE_ADS_CAMPAIGN_SHOPPING_PERFORMANCE_EPAGESJ_URL,
        "$SubTestName - Verify doGet calls correctly the epagesj endpoint."
    );
    is(
        $Response,
        EXPECTED_GOOGLE_ADS_CAMPAIGN_SHOPPING_PERFORMANCE_RESPONSE,
        "$SubTestName - Verify that campaign shopping performance response is as expected."
    );

}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------

sub _createSUT {
    my $self = shift;
    my ($hSUTParams) = @_;

    my $hServiceParams = {
        'oauth_code' => AUTH_CODE,
        'Shop' => $hSUTParams->{'Shop'} // undef,
    };

    my $Mockify = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService', [$hServiceParams]);

    $Mockify->mockImported('DE_EPAGES::GoogleSmartShopping::API::Tools', 'LogGoogleSmartShopping')
        ->whenAny()
        ->thenReturnUndef();

    if (defined($hSUTParams->{'UserAgent'})) {
        $Mockify->mockStatic('LWP::UserAgent::new')->whenAny()->thenReturn($hSUTParams->{'UserAgent'});
    }

    if (defined($hSUTParams->{'DoGetResponse'})) {
        $Mockify->mock('doGet')
            ->whenAny()
            ->thenReturn($hSUTParams->{'DoGetResponse'});
    }

    if (defined($hSUTParams->{'DoPostResponse'})) {
        $Mockify->mock('doPost')
            ->whenAny()
            ->thenReturn($hSUTParams->{'DoPostResponse'});
    }

    if (defined($hSUTParams->{'DoPutResponse'})) {
        $Mockify->mock('doPut')
            ->whenAny()
            ->thenReturn($hSUTParams->{'DoPutResponse'});
    }

    if (defined($hSUTParams->{'DoDeleteError'})) {
        $Mockify->mock('doDelete')
            ->whenAny()
            ->thenThrowError($hSUTParams->{'DoDeleteError'});
    } elsif (defined($hSUTParams->{'DoDeleteResponse'})) {
        $Mockify->mock('doDelete')
            ->whenAny()
            ->thenReturn($hSUTParams->{'DoDeleteResponse'});
    }

    if (defined($hSUTParams->{'Error'})) {
        $Mockify->mockImported('DE_EPAGES::Core::API::Error', 'ExistsError')
            ->whenAny()
            ->thenReturn(1);
        $Mockify->mockImported('DE_EPAGES::Core::API::Error', 'GetError')
            ->whenAny()
            ->thenReturn($hSUTParams->{'Error'});
    }


    $Mockify->mock('loadGoogleSettings')
        ->when()
        ->thenReturnUndef();

    $Mockify->mock('_getAdsAccountId')->when()->thenReturn(GOOGLE_ADS_ACCOUNT_ID);
    $Mockify->mock('_getAdsAccountName')->when()->thenReturn(GOOGLE_ADS_ACCOUNT_NAME);
    $Mockify->mock('_getAdsManagerAccountId')->when()->thenReturn(GOOGLE_ADS_MANAGER_ACCOUNT_ID);
    $Mockify->mock('_createAdminToken')->when()->thenReturn(ADMIN_ACCESS_TOKEN);
    $Mockify->mock('_getDeveloperToken')->when()->thenReturn(EPAGES_ADS_DEVELOPER_TOKEN);
    $Mockify->mock('_getEmail')->when()->thenReturn(ACCOUNT_EMAIL);
    $Mockify->mock('_getTimeZone')->when()->thenReturn(EPAGES_TZ);
    $Mockify->mock('_getCurrencyCode')->when()->thenReturn('EUR');

    return $Mockify->getMockObject();
}

sub _createGoogleConfig {
    my $self = shift;
    my ($hParams) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig', $hParams
    );

    return $Mockify->getMockObject();
}


sub _createShop {
    my $self = shift;
    my ($GoogleSmartShoppingConfig, $AlreadyExistingConfig) = @_;
    my $MockedAddress = $self->_createAddress();

    my $hAttributes = {
        'ID' => undef,
        'GoogleSmartShoppingConfig'   => $GoogleSmartShoppingConfig,
        'Address'                   => $MockedAddress,
        'LanguageID'                => 1,
        'NameOrAlias'               => 'ShopNameOrAlias',
        'WebUrlUnitySSL'            => 'WebUrlUnitySSL',
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    $MockifyShop->addMockWithReturnValueAndParameterCheck(
        'existsChild', $AlreadyExistingConfig, [ {'string' => 'GoogleSmartShoppingConfig'} ]
    );

    return $MockifyShop->getMockObject();
}

sub _createAddress {
    my $self = shift;

    my $hParameterList = {
        'Country'   => {'Code2' => TEST_ADDRESS_COUNTRY_ISO_CODE} ,
    };

    my $MockAddress = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Address::API::Object::Address',
        $hParameterList
    );

    return $MockAddress->getMockObject();
}


__PACKAGE__->runScript();

1;
