#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::UI_JSON
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for the GoogleSmartShopping JSON Dialog generation
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::UI_JSON;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify;
use Test::Mockify::Matcher qw ( HashRef Object String );
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled );

use Test::More;

use strict;

use constant CAMPAIGN_PRODUCTS_PAGE => {
    'results'     => 45, # total number of available items (not the amount returned)
    'page' => 1,
    'resultsPerPage' => 2,
    'items' => [
        { 'productName' => '01-Lifestyle Headphones', 'clicks' => '54', 'cost' => '14.00', 'conversions' => '2', 'conversionValue' => '35.00' },
        { 'productName' => '02-Navy Boots', 'clicks' => '34', 'cost' => '8.00', 'conversions' => '6', 'conversionValue' => '48.00' },
    ]
};

sub test {
    my $self = shift;

    $self->test_JSONSearchCampaignProductsMetrics();

    done_testing();
}

sub test_JSONSearchCampaignProductsMetrics {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $Shop = $self->_createShop();
    my $hFormQueryParameters = {
        'currentPage' => 1,
        'pageEntries' => 10,
        'pages'       => 5,
        'campaign'    => '434234687'
    };
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $DialogBuilder = $self->_createDialogBuilder();

    my $SUT = $self->_createSUT($DialogBuilder);

    $SUT->JSONSearchCampaignProductsMetrics($Servlet);

    ok( WasCalled($Servlet, 'form'), $SubTestName . ' - Verify that Servlet "form" method was called.' );
    ok( WasCalled($ServletForm, 'form'), $SubTestName . ' - Verify that "form" method was called form Servlet form.' );
    ok(
        WasCalled($DialogBuilder, 'fetchCampaignProductsPage'),
        $SubTestName . ' - Verify that campaign products page is gotten by using DialogBuilder module.'
    );
    ok(
        WasCalled($DialogBuilder, 'buildCampaignProductsDialog'),
        $SubTestName . ' - Verify that campaign products dialog data is built by using DialogBuilder module.'
    );

}

#----------------------------------------------------------------------------------------
# Mockery
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($DialogBuilder) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::UI::JSON');

    $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::DialogBuilder::new')
        ->when(String(), HashRef())
        ->thenReturn($DialogBuilder);

    $MockifySut->mock('writeContentOutput')
        ->whenAny()
        ->thenReturnUndef();

    return $MockifySut->getMockObject();
}

sub _createShop {
    my $self = shift;

    my $hAttributes = {};
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);

    return $MockifyShop->getMockObject();
}

sub _createServletForm {
    my $self = shift;
    my ($hQueryParameters) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::Presentation::API::Form');
    $Mockify->mock('form')
        ->when(Object(), String())
        ->thenReturn($hQueryParameters);

    return $Mockify->getMockObject();
}

sub _createServlet {
    my $self = shift;
    my ($Shop, $ServletForm) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);
    $MockifyServlet->addMockWithReturnValue('form', $ServletForm);
    $MockifyServlet->addMockWithReturnValue('languageID', 1);

    return $MockifyServlet->getMockObject();
}

sub _createDialogBuilder {
    my $self = shift;

    my $aParameterList = [];

    my $Mockify = Test::Mockify->new(
        'DE_EPAGES::GoogleSmartShopping::API::DialogBuilder',
        $aParameterList
    );
    $Mockify->mock('fetchCampaignProductsPage')
        ->when()
        ->thenReturn(CAMPAIGN_PRODUCTS_PAGE);
    $Mockify->mock('buildCampaignProductsDialog')
        ->when(HashRef())
        ->thenReturnUndef();

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();
