#==============================================================================
# package      DE_EPAGES::Installer::API::Constants
# state        public
# description  This file provides constants cartridge Installer
#==============================================================================
package DE_EPAGES::Installer::API::Constants;
use base qw(Exporter);
use strict;

our @EXPORT_OK = qw (
    FILENAME_INSTALL
    FILENAME_PATCH
    FILENAME_UNINSTALL
    FILENAME_CPAN

    STOPPED
    PENDING
    RUNNING
    DENIED
    MISSING

    START_PENDING
    STOP_PENDING
    CONTINUE_PENDING
    PAUSE_PENDING
    PAUSED

    INTERN_PUB_CARTRIDGE_PACK_SERVERS
    PUBLIC_CARTRIDGE_PACK_SERVERS
    INTERN_LANGUAGE_PACK_SERVERS
    PUBLIC_LANGUAGE_PACK_SERVERS
);

use constant FILENAME_INSTALL   => "$ENV{'EPAGES_CARTRIDGES'}/DE_EPAGES/Installer/Templates/InstallReasons.tmpl";
use constant FILENAME_PATCH     => "$ENV{'EPAGES_CARTRIDGES'}/DE_EPAGES/Installer/Templates/PatchReasons.tmpl";
use constant FILENAME_UNINSTALL => "$ENV{'EPAGES_CARTRIDGES'}/DE_EPAGES/Installer/Templates/UninstallReasons.tmpl";

#========================================================================================
# constants for service status
#
# global STOPPED | 1 - Service is stopped
#                   windows: any status but SERVICE_START_PENDING/SERVICE_RUNNING | int
# global PENDING | 2 - service is starting
#                   windows: SERVICE_START_PENDING; UNIX: not implemented | int
# global RUNNING | 4 - service is running
#                   windows: SERVICE_RUNNING | int
# global DENIED  | 8 - status unknown due to insufficient privileges | int
# global MISSING | 16 - service not installed | int
use constant STOPPED => 1;
use constant PENDING => 2;
use constant RUNNING => 4;
use constant DENIED  => 8;
use constant MISSING => 16;

#========================================================================================
# more constants for windows service status
#
# see:
# http://msdn.microsoft.com/en-us/library/system.serviceprocess.servicecontrollerstatus.aspx
use constant START_PENDING => 2;
use constant STOP_PENDING => 3;
use constant CONTINUE_PENDING => 5;
use constant PAUSE_PENDING => 6;
use constant PAUSED => 7;

#========================================================================================
# constants for epages repositories
use constant INTERN_PUB_CARTRIDGE_PACK_SERVERS => qw(
    ftp://ftp.epages.com/pub/epages/build/repo/epages
);

use constant PUBLIC_CARTRIDGE_PACK_SERVERS => qw(
    http://epages-software.de/repo/epages
);
use constant INTERN_LANGUAGE_PACK_SERVERS => qw(
    ftp://ftp.epages.com/pub/epages/build/repo/epages
);
use constant PUBLIC_LANGUAGE_PACK_SERVERS => qw(
    http://epages-software.de/repo/epages
);

1;
