use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::File qw (SetAccessRightsRecursive);
use DE_EPAGES::Core::API::Net qw (GetFile);
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Core::API::SystemCommand qw (UnpackTar);

my $DEFAULT_SOURCE = 'ftp://ftp.epages.com/pub/epages/build/repo/epages-intern/AntiquePatches.tgz';

sub usage {
    print <<END_USAGE;
Description:
  Adds DE_EPAGES/Patches/Patch_VERSION for 6.15.0 <= VERSION <= 7.4.0.

Usage:
  perl $0 [Options]

Options:
  -source URL : get antique patches package from URL instead of
   $DEFAULT_SOURCE
  -force      : add antique patches even if DE_EPAGES/.antique-patches exists
  -help       : show the command line options
END_USAGE
    exit 2;
}

sub Main {
    local $| = 1;

    my %Options = ();
    my @GetOpt = qw(source=s force help);

    usage() unless GetOptions(\%Options, @GetOpt);
    usage() if defined $Options{help};

    if (not $Options{force} and -f "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/.antique-patches") {
        print "File found: $ENV{EPAGES_CARTRIDGES}/DE_EPAGES/.antique-patches\n";
        print "Antique patches already exist, exiting.\n";
        exit 0;
    }

    my $time1 = time;

    # download AntiquePatches.tgz
    my $AntiquePatchesTgz = "$ENV{EPAGES_CARTRIDGES}/AntiquePatches.tgz";
    my $AntiquePatchesUrl = $Options{source} // $DEFAULT_SOURCE;
    print "GetFile($AntiquePatchesUrl, $AntiquePatchesTgz)\n";
    GetFile($AntiquePatchesUrl, $AntiquePatchesTgz);

    # unpack AntiquePatches.tgz
    print "UnpackTar($AntiquePatchesTgz, $ENV{EPAGES_CARTRIDGES})\n";
    UnpackTar($AntiquePatchesTgz, $ENV{EPAGES_CARTRIDGES});
    SetAccessRightsRecursive("$ENV{EPAGES_CARTRIDGES}/AntiquePatches",
                             {User => 'ep_appl', Group => 'epages'});

    print "cp -rlf $ENV{EPAGES_CARTRIDGES}/AntiquePatches $ENV{EPAGES_CARTRIDGES}/DE_EPAGES\n";
    system("cp -rlf $ENV{EPAGES_CARTRIDGES}/AntiquePatches $ENV{EPAGES_CARTRIDGES}/DE_EPAGES");

    print "rm -rf $ENV{EPAGES_CARTRIDGES}/AntiquePatches $AntiquePatchesTgz\n";
    system("rm -rf $ENV{EPAGES_CARTRIDGES}/AntiquePatches $AntiquePatchesTgz");

    my $time2 = time;
    printf "need %.1f seconds\n", $time2-$time1;
}

RunScript(
    'Sub' => \&Main
);

1;
