#========================================================================================
# package      DE_EPAGES::Order::t::unit::BasketCreator
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Order::API::BasketCreator
#========================================================================================
package DE_EPAGES::Order::t::unit::BasketCreator;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Order::API::BasketCreator;
use DE_EPAGES::Shop::API::Constants qw ( TAX_MODEL_GROSS );
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw (GetParametersFromMockifyCall);
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_createBasket();

    done_testing();
}



sub test_createBasket {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->_createShop();
    my $BasketCreator = $self->_createBasketCreator($Shop);

    my $hBasketData = {
        'currency' => 'EUR',
        'taxType' => 'GROSS',
        'locale' => 'de_DE'
    };

    my $Basket = $BasketCreator->createBasket($hBasketData);

    is($Basket->alias(), 'NewBasket', "$SubTestName - test if the new created basket was returned.");

    my $aCreateBasketCallParams = GetParametersFromMockifyCall($BasketCreator, '_CreateBasket');
    is($aCreateBasketCallParams->[0]->{'User'}->alias(), 'BasketUserAlias', "$SubTestName - test if basket was created with correct user.");

    my $AnonymousUsersFolder = $self->{'__objectsToCheck'}->{'AnonymousUsersFolder'};
    my $aInsertChildCallParams = GetParametersFromMockifyCall($AnonymousUsersFolder, 'insertChild');
    is($aInsertChildCallParams->[0], 'AnonymousUser', "$SubTestName - test if AnonymousUser was created.");
    is($aInsertChildCallParams->[1]->{'Alias'}, 'TestAlias', "$SubTestName - test if user was created with correct alias.");

    my $BasketLineItemContainer = $self->{'__objectsToCheck'}->{'BasketLineItemContainer'};
    my $aSetLineItemContainerCallParams = GetParametersFromMockifyCall($BasketLineItemContainer, 'setLineItemContainer');
    is_deeply($aSetLineItemContainerCallParams->[0], $self->_createExpectedLineItemContainerData(), "$SubTestName - test if setLineItemContainer was called with the correct parameter");
}



sub _createBasketCreator {
    my $self = shift;
    my ($Basket) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Order::API::BasketCreator',
        [$Basket]
    );
    $Mockify->addMock('_Transaction', sub {
        my $MockedSelf = shift;
        my ($cCode, $cRollback) = @_;
        $cCode->();
        return;
    });
    $Mockify->addMockWithReturnValueAndParameterCheck('_CreateBasket', $self->_createBasket(), ['hashref']);
    $Mockify->addMockWithReturnValueAndParameterCheck('_createBasketLineItemContainer', $self->_createBasketLineItemContainer($Basket), ['object']);
    return $Mockify->getMockObject();
}



sub _createShop {
    my $self = shift;
    my $hAttributes = {
        'DeliveryThresholdIsActive' => 0,
        'Address' => undef,
        'TaxModel' => TAX_MODEL_GROSS ,
        'TaxMatrix' => undef,
    };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    $Shop->mock('folder', sub {
        return $self->_createAnonymousUsersFolder();
    });
    return $Shop;
}



sub _createBasket {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'NewBasket',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::Basket', $hAttributes);
}



sub _createAnonymousUsersFolder {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Object::API::Object::Object',
        []
    );
    $Mockify->addMockWithReturnValueAndParameterCheck('nextAlias', 'TestAlias', ['string']);
    $Mockify->addMockWithReturnValueAndParameterCheck('insertChild', $self->_createBasketUser(), ['string', 'hashref']);
    my $AnonymousUsersFolder = $Mockify->getMockObject();
    $self->{'__objectsToCheck'}->{'AnonymousUsersFolder'} = $AnonymousUsersFolder;
    return $AnonymousUsersFolder;
}



sub _createBasketUser {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'BasketUserAlias',
    };
    my $BasketUser = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Permission::API::Object::User', $hAttributes);
    return $BasketUser;
}



sub _createBasketLineItemContainer {
    my $self = shift;
    my ($Basket) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Order::API::BasketLineItemContainer',
        [$Basket]
    );
    $Mockify->addMockWithReturnValueAndParameterCheck('setLineItemContainer', undef, ['hashref']);
    my $BasketLineItemContainer = $Mockify->getMockObject();
    $self->{'__objectsToCheck'}->{'BasketLineItemContainer'} = $BasketLineItemContainer;
    return $BasketLineItemContainer;
}



sub _createExpectedLineItemContainerData {
    return {
        'PaymentMethod' => undef,
        'ShippingMethod' => undef,
        'ProductLineItems' => [],
        'LocaleID' => 'de_DE',
        'TaxArea' => undef,
        'CurrencyID' => 'EUR',
        'TaxModel' => 'gross'
    };
}
__PACKAGE__->runScript();