package DE_EPAGES::Order::t::unit::SplitableTaxes;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Order::API::SplitableTaxes;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockery;
use DE_EPAGES::Shop::API::Constants qw ( TAX_MODEL_NET );
use Test::More;

use strict;


sub test {
    my $self = shift;

    $self->test_getPaymentData();
    $self->test_getShippingData();

    $self->test_getShippingDataNetShop();

    done_testing();
}

sub test_getPaymentData {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $LanguageID = '2';
    my $Formatter = $self->_createFormatter('Formatter');
    my $LineItemContainer = $self->_createLineItemContainer();
    my $SplitableTaxes = $self->_createSUT();
    my $aPaymentTaxes = $SplitableTaxes->calculatePaymentTax($LineItemContainer, $LanguageID, $Formatter);

    my $hExpectedPaymentData = $self->_expectedaTaxesPayment();
    is_deeply($aPaymentTaxes, $hExpectedPaymentData, "$SubTestName - test if expected payment tax data was returned.");
}

sub test_getShippingData {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $LanguageID = '2';
    my $Formatter = $self->_createFormatter('Formatter');
    my $LineItemContainer = $self->_createLineItemContainer();
    my $SplitableTaxes = $self->_createSUT();
    my $aShippingTaxes = $SplitableTaxes->calculateShippingTax($LineItemContainer, $LanguageID, $Formatter);

    my $hExpectedShippingData = $self->_expectedaTaxesShipping();
    is_deeply($aShippingTaxes, $hExpectedShippingData, "$SubTestName - test if expected shipping tax data was returned.");
}

sub test_getShippingDataNetShop {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $LanguageID = '2';
    my $Formatter = $self->_createFormatter('Formatter');
    my $LineItemContainer = $self->_createLineItemContainer();
    $LineItemContainer->set({'TaxModel' => TAX_MODEL_NET});
    $LineItemContainer->get('Shipping')->set({'LineItemPrice' => '16.81'});

    my $SplitableTaxes = $self->_createSUT();
    my $aShippingTaxes = $SplitableTaxes->calculateShippingTax($LineItemContainer, $LanguageID, $Formatter);

    my $hExpectedShippingData = $self->_expectedaTaxesShippingNetShop();
    is_deeply($aShippingTaxes, $hExpectedShippingData, "$SubTestName - test if expected shipping tax data was returned.");
}


sub _createSUT {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Order::API::SplitableTaxes', []);
    my $Counter = 0;
    $Mockify->addMock('_LoadObject', sub {
        my $aClassTax = [$self->_createTaxClass('.19'), $self->_createTaxClass('.07'), $self->_createTaxClassDigital('.19'), $self->_createTaxClass('0')];
        return @{$aClassTax}[$Counter++];
    });
    $Mockify->addMockWithReturnValueAndParameterCheck('_GetInfoCurrency', {'MinorUnit' => 2}, ['string']);
    $Mockify->addMockWithReturnValue('_createNumberFormat', $self->_createNumberFormat());
    return $Mockify->getMockObject();
}


sub _createNumberFormat{
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Core::API::Object::NumberFormat', []);
    $Mockify->addMock('round', sub {
        my $MockedSelf = shift;
        my @Parameters = @_;
        my $MethodName = 'round';
        my $aParameterTypes = ['string',{'int' => 2}];
        ## no critic (ProhibitPrivateSubroutineUse)
        $Mockify->_checkParameterTypesForMethod( $MethodName , $aParameterTypes );
        $Mockify->_storeParameters( $MethodName, $MockedSelf, \@Parameters );
        $Mockify->_testParameterTypes( $MethodName , $aParameterTypes, \@Parameters );
        $Mockify->_testParameterAmount( $MethodName , $aParameterTypes, \@Parameters );
        ## use critic (ProhibitPrivateSubroutineUse)
        my ($Number, $Precision)  = @Parameters; ## no critic (ProhibitUnusedVariablesEnhanced)
        return sprintf("%.2f", $Number);
        }
    );
    return $Mockify->getMockObject();
}

sub _createLineItemContainer {
    my $self = shift;
    my $MockifyObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', {
        'CurrencyID' => 'EUR',
        'PaymentDiscount' => $self->_createLineItemPaymentDiscount(),
        'Payment' => $self->_createLineItemPayment(),
        'Shipping' => $self->_createLineItemShipping(),
        'TaxModel' => '1',
        'TaxMatrix' => $self->_createTaxMatrix(),
        'TaxMatrixDigital' => $self->_createTaxMatrixDigital(),
        'TaxArea' => $self->_createTaxArea(),
        'TaxAreaDigital' => $self->_createTaxAreaDigital(),
    });
    $MockifyObject->addMockWithReturnValue('netPricesForTax', $self->mockedNetPricesForTax());
    return $MockifyObject->getMockObject();
}


sub _createShop {
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shop::API::Object::Shop', []);
    $Mockify->addMockWithReturnValueAndParameterCheck('siteLocale', 'en_GB', [{'string' => 'en_GB'}]);
    return $Mockify->getMockObject();
}

sub _createFormatter {
    my $self = shift;
    my $Shop = $self->_createShop();
    my $LocaleID = 'en_GB';
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shop::API::Resource::Formatter', [$Shop, $LocaleID]);
    $Mockify->addMock('price', sub {
        my $MockedSelf = shift;
        my ($Price, $Currency, $TaxModel) = @_;
        return $Price;
    });
    return $Mockify->getMockObject();
}

sub _createLineItemPaymentDiscount {
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::LineItemPaymentDiscount', {
        'LineItemPrice' => 5,
    });
    return $Mockify->getMockObject();
}

sub _createLineItemPayment {
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::LineItemPayment', {
            'LineItemPrice' => 5,
        });
    return $Mockify->getMockObject();
}

sub _createLineItemShipping {
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::LineItemShipping', {
        'LineItemPrice' => 20,
    });
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------------------------------
sub _createTaxClass {
    my $self = shift;
    my ($TaxRate) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxClass', {
        'Name' => 'TaxClassName ' . $TaxRate * 100,
        'instanceOf' => 'TaxClass',
        'GUID' => 'TaxClass GUID',
        'TaxRate' => $TaxRate, # this attribute does not exist in TaxClass, it belongs to taxMatrix and is calculate...
    });
    $Mockify->addMock('alias', sub {
        return $TaxRate;
    });
    return $Mockify->getMockObject();
}

sub _createTaxMatrix {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxMatrix', { });
    $Mockify->addMock('taxRate', sub {
        my $MockedSelf = shift;
        my @Parameters = @_;
        my $MethodName = 'taxRate';
        my $aParameterTypes = ['object','object'];
        ## no critic (ProhibitPrivateSubroutineUse)
        $Mockify->_checkParameterTypesForMethod( $MethodName , $aParameterTypes );
        $Mockify->_storeParameters( $MethodName, $MockedSelf, \@Parameters );
        $Mockify->_testParameterTypes( $MethodName , $aParameterTypes, \@Parameters );
        $Mockify->_testParameterAmount( $MethodName , $aParameterTypes, \@Parameters );
        ## use critic (ProhibitPrivateSubroutineUse)
        my ($TaxArea, $TaxClass)  = @Parameters; ## no critic (ProhibitUnusedVariablesEnhanced)
        return $TaxClass->get('TaxRate');
        }
    );
    $Mockify->addMock('child', sub {
        my $MockedSelf = shift;
        my @Parameters = @_;
        return $self->_createTaxClass($Parameters[0]);
    });
    $Mockify->addMock('gross2net', sub {
        my $MockedSelf = shift;
        my @Parameters = @_;
        my $MethodName = 'gross2net';
        my $aParameterTypes = ['string','string', 'hashref'];
        ## no critic (ProhibitPrivateSubroutineUse)
        $Mockify->_checkParameterTypesForMethod( $MethodName , $aParameterTypes );
        $Mockify->_storeParameters( $MethodName, $MockedSelf, \@Parameters );
        $Mockify->_testParameterTypes( $MethodName , $aParameterTypes, \@Parameters );
        $Mockify->_testParameterAmount( $MethodName , $aParameterTypes, \@Parameters );
        ## use critic (ProhibitPrivateSubroutineUse)
        my ($CurrencyId, $GrossPrice, $hTaxData)  = @Parameters; ## no critic (ProhibitUnusedVariablesEnhanced)
        return $GrossPrice / (1+ $hTaxData->{'TaxRate'});
        }
    );
    $Mockify->addMock('net2gross', sub {
        my $MockedSelf = shift;
        my @Parameters = @_;
        my $MethodName = 'net2gross';
        my $aParameterTypes = ['string','string', 'hashref'];
        ## no critic (ProhibitPrivateSubroutineUse)
        $Mockify->_checkParameterTypesForMethod( $MethodName , $aParameterTypes );
        $Mockify->_storeParameters( $MethodName, $MockedSelf, \@Parameters );
        $Mockify->_testParameterTypes( $MethodName , $aParameterTypes, \@Parameters );
        $Mockify->_testParameterAmount( $MethodName , $aParameterTypes, \@Parameters );
        ## use critic (ProhibitPrivateSubroutineUse)
        my ($CurrencyId, $GrossPrice, $hTaxData)  = @Parameters; ## no critic (ProhibitUnusedVariablesEnhanced)
        return $GrossPrice * (1+ $hTaxData->{'TaxRate'});
        }
    );
    return $Mockify->getMockObject();
}

sub _createTaxArea {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxArea', { });
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------------------------------
sub _createTaxClassDigital {
    my $self = shift;
    my ($TaxRate) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxClassDigital', {
        'Name' => 'TaxClassDigitalName ' . $TaxRate * 100,
        'instanceOf' => 'TaxClassDigital',
        'GUID' => 'TaxClassDigital GUID',
        'TaxRate' => $TaxRate, # this attribute does not exist in TaxClass, it belongs to taxMatrix and is calculate...
    });
    return $Mockify->getMockObject();
}

sub _createTaxMatrixDigital {
    my $self = shift;
    my ($TaxRate) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxMatrixDigital', { });
    $Mockify->addMock('taxRateDigital', sub {
        my $MockedSelf = shift;
        my @Parameters = @_;
        my $MethodName = 'taxRateDigital';
        my $aParameterTypes = ['object','object'];
        ## no critic (ProhibitPrivateSubroutineUse)
        $Mockify->_checkParameterTypesForMethod( $MethodName , $aParameterTypes );
        $Mockify->_storeParameters( $MethodName, $MockedSelf, \@Parameters );
        $Mockify->_testParameterTypes( $MethodName , $aParameterTypes, \@Parameters );
        $Mockify->_testParameterAmount( $MethodName , $aParameterTypes, \@Parameters );
        ## use critic (ProhibitPrivateSubroutineUse)

        my ($TaxArea, $TaxClass)  = @Parameters; ## no critic (ProhibitUnusedVariablesEnhanced)
        return $TaxClass->get('TaxRate');
        }
    );
    $Mockify->addMock('gross2net', sub {
        my $MockedSelf = shift;
        my @Parameters = @_;
        my $MethodName = 'gross2net';
        my $aParameterTypes = ['string','string', 'hashref'];
        ## no critic (ProhibitPrivateSubroutineUse)
        $Mockify->_checkParameterTypesForMethod( $MethodName , $aParameterTypes );
        $Mockify->_storeParameters( $MethodName, $MockedSelf, \@Parameters );
        $Mockify->_testParameterTypes( $MethodName , $aParameterTypes, \@Parameters );
        $Mockify->_testParameterAmount( $MethodName , $aParameterTypes, \@Parameters );
        ## use critic (ProhibitPrivateSubroutineUse)
        my ($CurrencyId, $GrossPrice, $hTaxData)  = @Parameters; ## no critic (ProhibitUnusedVariablesEnhanced)
        return $GrossPrice / (1+ $hTaxData->{'TaxRate'});
        }
    );
    $Mockify->addMock('net2gross', sub {
        my $MockedSelf = shift;
        my @Parameters = @_;
        my $MethodName = 'net2gross';
        my $aParameterTypes = ['string','string', 'hashref'];
        ## no critic (ProhibitPrivateSubroutineUse)
        $Mockify->_checkParameterTypesForMethod( $MethodName , $aParameterTypes );
        $Mockify->_storeParameters( $MethodName, $MockedSelf, \@Parameters );
        $Mockify->_testParameterTypes( $MethodName , $aParameterTypes, \@Parameters );
        $Mockify->_testParameterAmount( $MethodName , $aParameterTypes, \@Parameters );
        ## use critic (ProhibitPrivateSubroutineUse)
        my ($CurrencyId, $GrossPrice, $hTaxData)  = @Parameters; ## no critic (ProhibitUnusedVariablesEnhanced)
        return $GrossPrice * (1+ $hTaxData->{'TaxRate'});
        }
    );
    return $Mockify->getMockObject();
}

sub _createTaxAreaDigital {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxAreaDigital', { });
    return $Mockify->getMockObject();
}

sub mockedNetPricesForTax {
    my $self = shift;
    return {
        'Total' => {
          '4672' => '104.30303030303',
          '4673' => '52.1515151515152',
          '4752' => '417.212121212121',
          '4674' => '201.333333333333'
        },
        'LineItemShipping' => {
          '4672' => '3.63636363636364',
          '4673' => '1.81818181818182',
          '4752' => '14.5454545454545',
          '4674' => 0
        },
        'LineItemPaymentDiscount' => {
          'NoTaxClass' => 5
        },
        'LineItem' => {
          '4672' => 100,
          '4673' => 50,
          '4752' => 400,
          '4674' => 200
        },
    };
}

sub _expectedaTaxesPayment {
    my $self = shift;
    return [
        {
          'taxClass' => {
                          'taxClassId' => 'TaxClass GUID',
                          'name' => 'TaxClassName 19',
                          'percentage' => 19
                        },
          'price' => '0.21'
        },
        {
          'taxClass' => {
                          'taxClassId' => 'TaxClass GUID',
                          'name' => 'TaxClassName 7',
                          'percentage' => 7
                        },
          'price' => '0.04'
        },
        {
          'taxClass' => {
                          'taxClassId' => 'TaxClassDigital GUID',
                          'name' => 'TaxClassDigitalName 19',
                          'percentage' => 19
                        },
          'price' => '0.85'
        },
        {
          'taxClass' => {
                          'taxClassId' => 'TaxClass GUID',
                          'name' => 'TaxClassName 0',
                          'percentage' => 0
                        },
          'price' => '0'
        }
    ];
}


sub _expectedaTaxesShipping {
    my $self = shift;
    return [
        {
          'taxClass' => {
                          'taxClassId' => 'TaxClass GUID',
                          'name' => 'TaxClassName 19',
                          'percentage' => 19
                        },
          'price' => '0.58'
        },
        {
          'taxClass' => {
                          'taxClassId' => 'TaxClass GUID',
                          'name' => 'TaxClassName 7',
                          'percentage' => 7
                        },
          'price' => '0.12'
        },
        {
          'taxClass' => {
                          'taxClassId' => 'TaxClassDigital GUID',
                          'name' => 'TaxClassDigitalName 19',
                          'percentage' => 19
                        },
          'price' => '2.32'
        },
        {
          'taxClass' => {
                          'taxClassId' => 'TaxClass GUID',
                          'name' => 'TaxClassName 0',
                          'percentage' => 0
                        },
          'price' => '0'
        }
    ];
}

sub _expectedaTaxesShippingNetShop {
    my $self = shift;
    return [
        {
          'taxClass' => {
                          'taxClassId' => 'TaxClass GUID',
                          'name' => 'TaxClassName 19',
                          'percentage' => 19
                        },
          'price' => '0.58'
        },
        {
          'taxClass' => {
                          'taxClassId' => 'TaxClass GUID',
                          'name' => 'TaxClassName 7',
                          'percentage' => 7
                        },
          'price' => '0.11'
        },
        {
          'taxClass' => {
                          'taxClassId' => 'TaxClassDigital GUID',
                          'name' => 'TaxClassDigitalName 19',
                          'percentage' => 19
                        },
          'price' => '2.32'
        },
        {
          'taxClass' => {
                          'taxClassId' => 'TaxClass GUID',
                          'name' => 'TaxClassName 0',
                          'percentage' => 0
                        },
          'price' => '0'
        }
    ];
}

__PACKAGE__->runScript();