#========================================================================================
# package      DE_EPAGES::Product::t::unit::ProductService2
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Product::API::ProductService
#               testing methods:
#               - getProductData()
#========================================================================================
package DE_EPAGES::Product::t::unit::ProductService2;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Product::API::ProductService;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw (WasCalled);
use DE_EPAGES::Test::Mock::Mockery;
use Test::More;
use Test::MockModule;
use feature qw ( switch );

use strict;



sub test {
    my $self = shift;

    my $Module = Test::MockModule->new('DE_EPAGES::Product::API::ProductService');
    $Module->mock('TriggerHook', sub {
        return;
    });

    $self->test_getProductData();
    $self->test_getProductDetailsData();

    $Module->unmock('TriggerHook');

    $self->test_deleteProduct();

    done_testing();
}



sub test_getProductData {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $ProductService = $Mockery->get('SUT');
    my $Shop = $Mockery->get('Shop');
    my $Product = $Mockery->get('Product');

    my $LocaleID = 'de_DE';
    my $CurrencyID = 'EUR';

    my $hShopData = $ProductService->getShopData($Shop, $LocaleID, $CurrencyID);
    my $hProductData = $ProductService->getProductData($Product, $hShopData, undef, {'TaxModel' => 1});

    my $hExpectedProductData = $self->_createExpectedProductData();

    is_deeply($hProductData, $hExpectedProductData, "$SubTestName - test if expected data was returned.");
}



sub test_getProductDetailsData {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $ProductService = $Mockery->get('SUT');
    my $Shop = $Mockery->get('Shop');
    my $Product = $Mockery->get('Product');

    my $LocaleID = 'de_DE';
    my $CurrencyID = 'EUR';

    my $hProductData = $ProductService->getProductDetailsData($Shop, $Product, $LocaleID, $CurrencyID);

    my $hExpectedProductData = $self->_createExpectedProductDetailsData();
    is_deeply($hProductData, $hExpectedProductData, "$SubTestName - test if expected data was returned.");
}



sub test_deleteProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $ProductService = $Mockery->get('SUT');
    my $Product = $Mockery->get('Product');

    $ProductService->deleteProduct($Product);

    ok(WasCalled($Product, 'delete'), "$SubTestName - test if delete was called on the product object");
}




sub _createMocks {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['Formatter', 'Locale', 'Context'], sub {
        my ($Formatter, $Locale, $Context) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Product::API::ProductService', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createFormatter', $Formatter, ['object', 'string']);
        $Mockify->addMockWithReturnValueAndParameterCheck('_loadLocale', $Locale, ['string']);
        $Mockify->addMockWithReturnValueAndParameterCheck('_getShopContext', $Context, ['object']);
        return $Mockify->getMockObject();
    });
    $Mockery->register('Formatter', ['Shop'], sub {
        my ($Shop) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shop::API::Resource::Formatter', [$Shop, 'de_DE']);
        $Mockify->addMock('price', sub {
            my $MockedSelf = shift;
            my ($Amount) = @_;
            return $Amount;
        });
        $Mockify->addMock('quantity', sub {
            my $MockedSelf = shift;
            my ($Amount) = @_;
            return $Amount;
        });
        $Mockify->addMock('bool', sub {
            my $MockedSelf = shift;
            my ($Amount) = @_;
            return $Amount;
        });
        return $Mockify->getMockObject();
    });
    $Mockery->register('Locale', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Locale', {
            'LanguageID' => '1',
        });
        return $Mockify->getMockObject();
    });
    $Mockery->register('Context', ['Shop'], sub {
        my ($Shop) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shop::API::Resource::ShopRequestContext', [$Shop]);
        $Mockify->addMockWithReturnValue('apiBaseUrl', 'http://api.url.de');
        $Mockify->addMockWithReturnValue('imageBaseUrl', 'http://image.url.de');
        $Mockify->addMockWithReturnValueAndParameterCheck('validCurrency', 'EUR', [{'string'=>'EUR'}]);
        return $Mockify->getMockObject();
    });
    $Mockery->register('Shop', ['Product', 'TaxMatrix'], sub {
        my ($Product, $TaxMatrix) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
            'Products'                  => [ $Product ],
            'TaxMatrix'                 => $TaxMatrix,
            'TaxModel'                  => 1,
            'UseProductZoom'            => 'true',
            'Address'                   => undef,
            'DeliveryThresholdIsActive' => 1,
        });
        $Mockify->addMockWithReturnValueAndParameterCheck('siteLocale', undef, [{'string' => 'de_DE'}]);
        return $Mockify->getMockObject();
    });
    $Mockery->register('Product', ['Category', 'TaxClass', 'Site'], sub {
        my ($Category, $TaxClass, $Site) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'Site' => $Site,
            'Width'=> '100',
            'Height'=>'110',
            'Length'=>'120',
            'StockLevelAlert'=>'3',
            'ManufacturerSKU'=>'model-no-xyz',
            'DeliveryPeriod' => '10',
            'AvailabilityText' => '',
            'AllProductCategories' => [$Category],
            'AvailabilityIcon' => '',
            'IsMaster' => '',
            'SuperProduct' => '',
            'VariationTypeValues' => [],
            'IsAvailable' => 1,
            'IsVisible' => 1,
            'Description' => 'product description',
            'DisplayTitle' => 'Title',
            'IsSpecialOffer' => '0',
            'Weight' => '123',
            'WeightUnit' => 'kg',
            'Text' => 'a product text',
            'NameOrAlias' => 'name or alias of the product',
            'ImageHotDeal' => 'TheProductImage_hotdeal.png',
            'ImageMedium' => 'TheProductImage_medium.png',
            'ImageLarge' => 'https://www.google.de/images/branding/googlelogo/2x/googlelogo_color_272x92dp.png',
            'ImageSmall' => 'TheProductImage_small.png',
            'ImageThumbnail' => 'TheProductImage_thumb.png',
            'Path' => '/the/product/path/',
            'GUID' => '1236-363-3223324',
            'PriceQuantity' => '',
            'OrderUnit' => '',
            'RefUnit' => '',
            'StockLevel' => '51',
            'RefAmount' => '',
            'RefContentAmount' => '',
            'ShippingMethods' => [],
            'WebPath' => 'http://webpath.test.url',
            'WebUrl' => 'http://web.test.url',
            'EnergyLabelsString' => undef,
            'EnergyLabelHighestEfficiencyClass' => undef,
            'EnergyLabelSourceFile' => '',
            'ProductDataSheet' => '',
            'Alias' => 'AProduct',
            'UPCEAN' => '123-456-678',
            'Manufacturer' => 'the manufacturer of this product',
            'MainCharacteristics' => 'some main characteristics',
            'Keywords' => 'key, words, tags',
            'UserDefinedAttributeValues' => [],
            'ImagesSlideShowString' => 'someString with image name;',
            'TaxClass' => $TaxClass,
            'TaxClassDigital' => undef,
            'CrossSelling' => [],
            'ProductVideoProvider' => '[{"VideoType":"YouTube","VideoID":421964906}]',
            'ImageInformations' => {},
        });
        ## no critic (ProhibitMagicNumbers)
        $Mockify->addMockWithReturnValueAndParameterCheck('getManufacturerPrice', 23.4, [{'string'=>'EUR'}, {'int'=>1}]);
        $Mockify->addMockWithReturnValueAndParameterCheck('getEcoParticipation', 34.5, [{'string'=>'EUR'}, {'int'=>1}]);
        $Mockify->addMockWithReturnValueAndParameterCheck('getDeposit', 45.6, [{'string'=>'EUR'}, {'int'=>1}]);
        $Mockify->addMockWithReturnValueAndParameterCheck('getReferencePrice', undef, [{'string'=>'EUR'}, 'any']);
        $Mockify->addMockWithReturnValueAndParameterCheck('lowestSubProductPrice', undef, [{'string'=>'EUR'}, 'any']);
        $Mockify->addMockWithReturnValueAndParameterCheck('highestSubProductPrice', undef, [{'string'=>'EUR'}, 'any']);
        ## use critic (ProhibitMagicNumbers)
        $Mockify->addMockWithReturnValue('isMaster', 1);
        $Mockify->addMockWithReturnValue('existSubProductPrices', 0);
        $Mockify->addMockWithReturnValue('delete', 0);

        $Mockify->mock('getListPrice', sub {
            my $MockedSelf = shift;
            my ($CurrencyId, $TaxModel) = @_;
            return '12.3';
        });


        return $Mockify->getMockObject();
    });
    #-- Shop references Product, Product references Shop, how do we get around circular register with Mockery?
    #-- I have no idea, but luckily Product includes a Site attribute, so let's use that instead
    #--
    $Mockery->register('Site', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
            'IsUnityRegistered' => 0,
            'DeliveryThresholdIsActive' => 0,
        });
        $Mockify->addMockWithReturnValueAndParameterCheck('siteLocale', undef, [{'string' => 'de_DE'}]);
        return $Mockify->getMockObject();
    });
    $Mockery->register('Category', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Content::API::Object::Category', {
            'NavigationString' => 'MyCategoryNavigation',
            'GUID' => 'MyCategoryGUID',
        });
        return $Mockify->getMockObject();
    });
    $Mockery->register('TaxMatrix', ['TaxArea'], sub {
        my ($TaxArea) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxMatrix', {});
        $Mockify->addMockWithReturnValue('defaultTaxArea', $TaxArea);
        $Mockify->addMockWithReturnValueAndParameterCheck('taxRate', '0.19', [{'object'=>'DE_EPAGES::Tax::API::Object::TaxArea'}, {'object'=>'DE_EPAGES::Tax::API::Object::TaxClass'}]);

        my $TaxClass =  DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxClass', {
            'GUID' => '32525-3463-54524-545',
            'NameOrAlias' => 'TaxClass',
            'Alias' => 'TaxClass',
        })->getMockObject();
        $Mockify->addMockWithReturnValueAndParameterCheck('getTaxClassByAlias', $TaxClass, [{'string'=>'TaxClass'}]   );
        return $Mockify->getMockObject();
    });


    $Mockery->register('TaxClass', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxClass', {
            'GUID' => '32525-3463-54524-545',
            'NameOrAlias' => 'TaxClass',
            'Alias' => 'TaxClass',
        });
        return $Mockify->getMockObject();
    });
    $Mockery->register('TaxArea', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxArea', {});
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}



sub _createExpectedProductData {
    return {
        'productWidth'=>'100',
        'productLength'=>'120',
        'productHeight'=>'110',
        'minStocklevel'=>'3',
        'manufacturerProductNumber'=>'model-no-xyz',
        'energyLabelSourceFile' => undef,
        'visible' => 1,
        'priceInfo' => {
            'ecoParticipationPrice' => '34.5',
            'depositPrice' => '45.6',
            'basePrice' => undef,
            'quantity' => '',
            'manufacturerPrice' => '23.4',
            'priceWithDeposits' => '92.4',
            'lowestPrice' => undef,
            'highestPrice' => undef,
            'price' => '12.3',
            'taxClass' => {
                'taxClassId' => '32525-3463-54524-545',
                'name' => 'TaxClass',
                'percentage'=> '19',
            },
        },
        'energyLabelsString' => undef,
        'energyLabelsMaxEfficiencyString' => undef,
        'availability' => '',
        'sfUrl' => 'http://web.test.url',
        'deliveryPeriod' => '10',
        'deliveryWeight' => '123',
        'videos' => [
            {
                'source' => '421964906',
                'type' => 'YouTube'
            }
        ],
        'links' => [
            {
                'rel' => 'self',
                'href' => 'http://api.url.de/api/products/1236-363-3223324'
            },
            {
                'rel' => 'variations',
                'href' => 'http://api.url.de/api/products/1236-363-3223324/variations'
            },
            {
                'rel' => 'categories',
                'href' => 'http://api.url.de/api/categories/?productId=1236-363-3223324'
            }
        ],
        'searchKeywords' => [
            'key',
            'words',
            'tags'
        ],
        'upcEan' => '123-456-678',
        'availabilityText' => '',
        'productImage' => 'https://www.google.de/images/branding/googlelogo/2x/googlelogo_color_272x92dp.png',
        'images' => [
            {

                'url' => 'http://image.url.dehttp://webpath.test.url/TheProductImage_hotdeal.png',
                'classifier' => 'HotDeal',
                'isZoomEnabled' => 'true',
                'width' => undef,
                'height' => undef,
                'fileSize' => undef
            },
            {
                'url' => 'http://image.url.dehttp://webpath.test.url/TheProductImage_medium.png',
                'classifier' => 'Medium',
                'isZoomEnabled' => 'true',
                'width' => undef,
                'height' => undef,
                'fileSize' => undef

            },
            {
                'url' => 'https://www.google.de/images/branding/googlelogo/2x/googlelogo_color_272x92dp.png',
                'classifier' => 'Large',
                'isZoomEnabled' => 'true',
                'width' => undef,
                'height' => undef,
                'fileSize' => undef
            },
            {
                'url' => 'http://image.url.dehttp://webpath.test.url/TheProductImage_small.png',
                'classifier' => 'Small',
                'isZoomEnabled' => 'true',
                'width' => undef,
                'height' => undef,
                'fileSize' => undef
            },
            {
                'url' => 'http://image.url.dehttp://webpath.test.url/TheProductImage_thumb.png',
                'classifier' => 'Thumbnail',
                'isZoomEnabled' => 'true',
                'width' => undef,
                'height' => undef,
                'fileSize' => undef
            }
        ],
        'name' => 'name or alias of the product',
        'productVariationType' => 'regular',
        'productVariationMasterName' => undef,
        'productVariationMasterId' => undef,
        'productVariationSelection' => undef,
        'productNumber' => 'AProduct',
        'productId' => '1236-363-3223324',
        'essentialFeatures' => 'some main characteristics',
        'description' => 'a product text',
        'title' => 'Title',
        'forSale' => 0,
        'specialOffer' => '0',
        'manufacturer' => 'the manufacturer of this product',
        'shortDescription' => 'product description',
        'productDataSheet' => undef,
        'stocklevel' => 51,
    };
}


sub _createExpectedProductDetailsData {
    my $self = shift;
    my $hProductData = $self->_createExpectedProductData();

    $hProductData->{'shippingMethodsRestrictedTo'} = undef;
    push @{$hProductData->{'links'}}, {
        'rel' => 'slideshow',
        'href' => 'http://api.url.de/api/products/1236-363-3223324/slideshow'
    };
    push @{$hProductData->{'links'}}, {
        'rel' => 'main-category',
        'href' => 'http://api.url.de/api/categories/MyCategoryGUID',
        'title' => 'MyCategoryNavigation'
    };

    return $hProductData;
}
__PACKAGE__->runScript();