#========================================================================================
# package      DE_EPAGES::ProductPortal::Hooks::Shop
# state        public
# description  Imports default shop data into a new shop when it is created.
#========================================================================================
package DE_EPAGES::ProductPortal::Hooks::Shop;

use strict;
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias LoadObject);

#========================================================================================
# function     OnDeleteFeature
#----------------------------------------------------------------------------------------
# syntax       OnDeleteFeature( $hParams );
#----------------------------------------------------------------------------------------
# description  removes portals if feature was deleted.
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of tax class, key is
#               <ul>
#                   <li>Object - feature - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteFeature {
    my ($hParams) = @_;
    my $Feature = $hParams->{'Object'};
    $_->delete foreach @{LoadClassByAlias('Portal')->getObjects('ByFeatureID', [$Feature->id])};
}

#========================================================================================
# function     OnDeleteShopCurrency
#----------------------------------------------------------------------------------------
# syntax       OnDeleteShopCurrency( $hParams );
#----------------------------------------------------------------------------------------
# description  Add servlet param with connected product portals which are associated
#               with the given currencies
#----------------------------------------------------------------------------------------
# input        $hParams
#               <ul>
#                   <li>Currencies  | ids of the shop currency to be deleted
#                   <li>Servlet
#                   <li>HookCount
#                   <li>HookCounter
#                   <li>HookName
#                   <li>ShopID      | id of the shop
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteShopCurrency {
    my ($hParams) = @_;

    my $Shop = LoadObject($hParams->{'ShopID'});
    my $Servlet = $hParams->{'Servlet'};
    my $aPortalConfigs = $Shop->get('PortalConfigs');

    my @Portals;
    foreach my $PortalConfig (@$aPortalConfigs) {
        my $PortalConfigCurrencyID = $PortalConfig->get('CurrencyID');
        foreach my $CurrencyID ($hParams->{'Currencies'}) {
            push @Portals, $PortalConfig->get('HistoryName') if $PortalConfigCurrencyID eq $CurrencyID
        }
    }

    my $PortalsString = join ", ", @Portals;
    $Servlet->params( 'DONT_DELETE_CURRENCY', $PortalsString ) if scalar @Portals > 0;
    return;
}

1;
