#========================================================================================
# package      DE_EPAGES::ShopMigration::API::ShopTypeMapping
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the functions regarding shoptypemapping for
#               migration shop and its prepshop.
#========================================================================================
package DE_EPAGES::ShopMigration::API::ShopTypeMapping;
use base Exporter;
use strict;
use 5.010; # Needed because we use state vars

our @EXPORT_OK = qw(
    GetPrepShopFeaturesByShopTypeRef
    GetShopTypeMappingForShop
);

use DE_EPAGES::Object::API::Factory qw(LoadRootObject LoadObject LoadClassByAlias);
use DE_EPAGES::ShopMigration::API::Cartridge;
##no critic (RequireCorrectImports)
use DE_EPAGES::ShopMigration::API::Constants qw(
    @SHOPMIGRATION_PREPSHOP_FEATURES
    %SHOPMIGRATION_SHOPTYPE_MAPPING
);

#========================================================================================
# function     GetShopTypeMappingForShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetShopTypeMappingForShop($Shop);
#----------------------------------------------------------------------------------------
# description  Return mapping for shoptyperef of base and prepshop.
#----------------------------------------------------------------------------------------
# input        $Shop | base shop - shop object | object
#----------------------------------------------------------------------------------------
# return       $ShopTypeRefFolder | base shop - shoptyperef object | object
#========================================================================================
sub GetShopTypeMappingForShop {
    my ($Shop) = @_;

    my $ShopTypeRefFolder = LoadRootObject()->folder('ShopTypeRefs');
    my $ShopTypeRef = $Shop->get('ShopTypeRef');
    my $ShopTypeRefAlias = defined $ShopTypeRef ? $ShopTypeRef->alias : '_default';
    if (exists $SHOPMIGRATION_SHOPTYPE_MAPPING{$ShopTypeRefAlias}) {
        $ShopTypeRefAlias = $SHOPMIGRATION_SHOPTYPE_MAPPING{$ShopTypeRefAlias};
    } else {
        $ShopTypeRefAlias = $SHOPMIGRATION_SHOPTYPE_MAPPING{'_default'};
    }
    return $ShopTypeRefFolder->child($ShopTypeRefAlias);
}

#========================================================================================
# function     GetPrepShopFeaturesByShopTypeRef
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetPrepShopFeaturesByShopTypeRef($Shop);
#----------------------------------------------------------------------------------------
# description  Return mapping for shoptyperef of base and prepshop.
#----------------------------------------------------------------------------------------
# input        $Shop | base shop - shop object | object
#----------------------------------------------------------------------------------------
# return       $aFeatureValues | array of feature values | array
#========================================================================================
sub GetPrepShopFeaturesByShopTypeRef {
    my ($ShopTypeRef) = @_;
    my $aShopIDs = LoadClassByAlias('Shop')->findObjects('ByShopTypeRefID', [ $ShopTypeRef->id ]);
    my $FirstShopWithShopTypeRef = scalar(@$aShopIDs) ? LoadObject($aShopIDs->[0]) : undef;
    my $aFeatureValues;
    if (defined $FirstShopWithShopTypeRef) {
        $aFeatureValues = $FirstShopWithShopTypeRef->get('FeatureValues');
        foreach my $hFeature (@$aFeatureValues) {
            $hFeature->{'Alias'} = $hFeature->{'Feature'}->alias;
            $hFeature->{'FeatureID'} = $hFeature->{'Feature'}->id;
        }
    } else {
        $aFeatureValues = \@SHOPMIGRATION_PREPSHOP_FEATURES;
    }
    return $aFeatureValues;
}

1;
